# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["WorkflowItem"]


class WorkflowItem(BaseModel):
    workflow_name: str

    request_user_input: Optional[Literal["never", "maybe", "always"]] = None
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """

    workflow_alias: Optional[str] = None

    workflow_inputs: Optional[Dict[str, Union[str, Dict[str, Union[str, object]]]]] = None

    workflow_save_outputs: Optional[Dict[str, str]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .node_item_input_param import NodeItemInputParam
from .unary_condition_param import UnaryConditionParam
from .plan_config_input_param import PlanConfigInputParam
from .workflow_config_input_param import WorkflowConfigInputParam
from .compound_condition_input_param import CompoundConditionInputParam

__all__ = [
    "ConfigCreateParams",
    "Config",
    "ConfigStateMachineConfigInput",
    "ConfigStateMachineConfigInputMachine",
    "ConfigStateMachineConfigInputMachineNextNode",
    "ConfigStateMachineConfigInputMachineNextNodeCase",
    "ConfigStateMachineConfigInputMachineNextNodeCaseCondition",
]


class ConfigCreateParams(TypedDict, total=False):
    config: Required[Config]
    """Representation of a plan, i.e. a composition of workflows and branch complexes

    Public attributes: workflows: maps a workflow "name" to either a workflow yaml
    file or an inline workflow definition plan: representation of the graph
    connecting workflows / branch complexes

    Private attributes: helper_workflows: a list of workflows created by default, in
    order to support loops and other control flow features
    """


ConfigStateMachineConfigInputMachineNextNodeCaseCondition: TypeAlias = Union[
    UnaryConditionParam, CompoundConditionInputParam
]


class ConfigStateMachineConfigInputMachineNextNodeCase(TypedDict, total=False):
    condition: Required[ConfigStateMachineConfigInputMachineNextNodeCaseCondition]
    """Representation of a boolean function with a single input e.g.

    the condition specified by input_name: x operator: 'contains' ref_value: 'c'
    would evaluate to True if x == 'cat' Operators are defined in the constant
    function store CONDITIONAL_ACTION_MAP
    """

    value: Required[str]


class ConfigStateMachineConfigInputMachineNextNode(TypedDict, total=False):
    default: Required[str]

    cases: Iterable[ConfigStateMachineConfigInputMachineNextNodeCase]


class ConfigStateMachineConfigInputMachine(TypedDict, total=False):
    next_node: Required[ConfigStateMachineConfigInputMachineNextNode]
    """A switch statement for state machines to select the next state to execute."""

    workflow_config: Required[WorkflowConfigInputParam]

    write_to_state: Dict[str, str]


class ConfigStateMachineConfigInput(TypedDict, total=False):
    machine: Required[Dict[str, ConfigStateMachineConfigInputMachine]]

    starting_node: Required[str]

    id: Optional[str]

    account_id: Optional[str]

    application_variant_id: Optional[str]

    base_url: Optional[str]

    concurrency_default: Optional[bool]

    datasets: Optional[Iterable[object]]

    done_string: str

    egp_api_key_override: Optional[str]

    egp_ui_evaluation: Optional[object]

    evaluations: Optional[Iterable[NodeItemInputParam]]

    final_output_nodes: Optional[List[str]]

    initial_state: object

    nodes_to_log: Union[str, List[str], None]

    num_workers: Optional[int]

    streaming_nodes: Optional[List[str]]

    type: Optional[Literal["workflow", "plan", "state_machine"]]


Config: TypeAlias = Union[PlanConfigInputParam, WorkflowConfigInputParam, ConfigStateMachineConfigInput]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = [
    "NodeItem",
    "Config",
    "ConfigNodeConfig",
    "ConfigJinjaNodeConfig",
    "ConfigJinjaNodeConfigDataTransformations",
    "ConfigJinjaNodeConfigOutputTemplate",
    "ConfigChunkEvaluationNodeConfig",
    "ConfigRerankerNodeConfig",
    "ConfigRetrieverNodeConfig",
    "ConfigCitationNodeConfig",
    "ConfigCitationNodeConfigCitationContext",
    "ConfigSearchCitationNodeConfig",
    "ConfigDataTransformNodeConfig",
    "ConfigCreateMessagesNodeConfig",
    "ConfigCreateMessagesNodeConfigMessageConfig",
    "ConfigCreateMessagesNodeConfigMessageConfigAlternatingRoleMessages",
    "ConfigCreateMessagesNodeConfigMessageConfigSingleRoleMessages",
    "ConfigInsertMessagesConfig",
    "ConfigRemoveMessageConfig",
    "ConfigTokenizerChatTemplateConfig",
    "ConfigLlmEngineNodeConfig",
    "ConfigLlmEngineNodeConfigBatchSysKwargs",
    "ConfigResponseParserNodeConfig",
    "ConfigProcessingNodeConfig",
    "ConfigProcessingNodeConfigFunctionSpecs",
    "ConfigSqlExecutorNodeConfig",
    "ConfigStaticNodeConfig",
    "ConfigGenerationNodeConfig",
]


class ConfigNodeConfig(BaseModel):
    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None


class ConfigJinjaNodeConfigDataTransformations(BaseModel):
    jinja_helper_functions: Optional[List[Union[str, object]]] = None

    jinja_template_path: Optional[str] = None

    jinja_template_str: Optional[str] = None
    """Raw template to apply to the data.

    This should be a Jinja2 template string. Please note, the data will be mapped as
    'value' in the template. Default None corresponds to {{value}}. Should access
    property `jinja_template_str` or field `jinja_template_str_loaded` for the
    loaded template data
    """

    jinja_template_str_loaded: Optional[str] = None
    """
    The original jinja_template_str field from the config might not contain the
    needed template, and we may need to load S3 data specified with
    `jinja_template_path`. This field caches the loaded template content, it is also
    accessed through property `jinja_template_str`.
    """


class ConfigJinjaNodeConfigOutputTemplate(BaseModel):
    jinja_helper_functions: Optional[List[Union[str, object]]] = None

    jinja_template_path: Optional[str] = None

    jinja_template_str: Optional[str] = None
    """Raw template to apply to the data.

    This should be a Jinja2 template string. Please note, the data will be mapped as
    'value' in the template. Default None corresponds to {{value}}. Should access
    property `jinja_template_str` or field `jinja_template_str_loaded` for the
    loaded template data
    """

    jinja_template_str_loaded: Optional[str] = None
    """
    The original jinja_template_str field from the config might not contain the
    needed template, and we may need to load S3 data specified with
    `jinja_template_path`. This field caches the loaded template content, it is also
    accessed through property `jinja_template_str`.
    """


class ConfigJinjaNodeConfig(BaseModel):
    context_chunks_key: Optional[str] = None

    data_transformations: Optional[Dict[str, ConfigJinjaNodeConfigDataTransformations]] = None

    llm_model: Optional[str] = None

    log_output: Optional[bool] = None

    log_prefix: Optional[str] = None

    max_tokens: Optional[int] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    output_template: Optional[ConfigJinjaNodeConfigOutputTemplate] = None
    """
    Base model for a Jinja template. Guaranteed to store a string that can be read
    in to Template().
    """

    verbose: Optional[bool] = None


class ConfigChunkEvaluationNodeConfig(BaseModel):
    top_k_thresholds: List[int]

    fuzzy_match_threshold: Optional[float] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    require_all: Optional[bool] = None


class ConfigRerankerNodeConfig(BaseModel):
    num_to_return: int

    scorers: List[object]

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    score_threshold: Optional[float] = None


class ConfigRetrieverNodeConfig(BaseModel):
    num_to_return: int

    exact_knn_search: Optional[bool] = None

    knowledge_base_id: Optional[str] = None

    knowledge_base_name: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None


class ConfigCitationNodeConfigCitationContext(BaseModel):
    generate_with_llm: Optional[bool] = None

    metric: Optional[str] = None

    min_similarity: Optional[float] = None

    score: Optional[Literal["precision", "recall", "fmeasure"]] = None


class ConfigCitationNodeConfig(BaseModel):
    citation_type: Literal["rouge", "model_defined"]

    citation_context: Optional[ConfigCitationNodeConfigCitationContext] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    s3_path_override: Optional[str] = None


class ConfigSearchCitationNodeConfig(BaseModel):
    end_search_regex: str

    search_regex: str

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None


class ConfigDataTransformNodeConfig(BaseModel):
    action: str

    additional_inputs: Optional[object] = None

    apply_to_dictlist_leaves: Optional[bool] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None


class ConfigCreateMessagesNodeConfigMessageConfigAlternatingRoleMessages(BaseModel):
    role_value_pairs: List[Dict[str, str]]


class ConfigCreateMessagesNodeConfigMessageConfigSingleRoleMessages(BaseModel):
    content: str

    role: str


ConfigCreateMessagesNodeConfigMessageConfig: TypeAlias = Union[
    ConfigCreateMessagesNodeConfigMessageConfigAlternatingRoleMessages,
    ConfigCreateMessagesNodeConfigMessageConfigSingleRoleMessages,
]


class ConfigCreateMessagesNodeConfig(BaseModel):
    message_configs: List[ConfigCreateMessagesNodeConfigMessageConfig]

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None


class ConfigInsertMessagesConfig(BaseModel):
    index: int

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None


class ConfigRemoveMessageConfig(BaseModel):
    index: int

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None


class ConfigTokenizerChatTemplateConfig(BaseModel):
    llm_model: str

    add_generation_prompt: Optional[bool] = None

    kwargs: Optional[object] = None

    max_length: Optional[int] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    padding: Optional[bool] = None

    truncation: Optional[bool] = None


class ConfigLlmEngineNodeConfigBatchSysKwargs(BaseModel):
    checkpoint_path: Optional[str] = None

    labels: Optional[Dict[str, str]] = None

    num_shards: Optional[int] = None

    seed: Optional[int] = None


class ConfigLlmEngineNodeConfig(BaseModel):
    llm_model: str

    batch_run_mode: Optional[Literal["sync", "async"]] = None

    batch_sys_kwargs: Optional[ConfigLlmEngineNodeConfigBatchSysKwargs] = None

    frequency_penalty: Optional[float] = None

    guided_choice: Optional[List[str]] = None

    guided_json: Optional[object] = None

    guided_regex: Optional[str] = None

    include_stop_str_in_output: Optional[bool] = None

    max_tokens: Optional[int] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    presence_penalty: Optional[float] = None

    stop_sequences: Optional[List[str]] = None

    temperature: Optional[float] = None

    timeout: Optional[int] = None

    top_k: Optional[int] = None

    top_p: Optional[float] = None


class ConfigResponseParserNodeConfig(BaseModel):
    action: str

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    reference_value: Optional[object] = None


class ConfigProcessingNodeConfigFunctionSpecs(BaseModel):
    kwargs: object

    path: str


class ConfigProcessingNodeConfig(BaseModel):
    function_specs: Dict[str, ConfigProcessingNodeConfigFunctionSpecs]

    return_key: str

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None


class ConfigSqlExecutorNodeConfig(BaseModel):
    connector_kwargs: Dict[str, str]

    connector_type: Optional[Literal["snowflake"]] = None

    log_queries: Optional[bool] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    return_type: Optional[Literal["df", "dicts", "markdown", "json", "str"]] = None

    schema_remapping_file: Optional[str] = None

    secrets: Optional[List[str]] = None


class ConfigStaticNodeConfig(BaseModel):
    from_file: Union[List[object], str, object, None] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    value: Optional[object] = None


class ConfigGenerationNodeConfig(BaseModel):
    llm_model: Optional[str] = None

    llm_model_deployment: Optional[str] = None

    llm_model_instance: Optional[str] = None

    max_tokens: Optional[int] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    stop_sequences: Optional[List[str]] = None

    strip_whitespace: Optional[bool] = None

    temperature: Optional[float] = None

    tool_name: Optional[str] = None


Config: TypeAlias = Union[
    ConfigNodeConfig,
    ConfigJinjaNodeConfig,
    ConfigChunkEvaluationNodeConfig,
    ConfigRerankerNodeConfig,
    ConfigRetrieverNodeConfig,
    ConfigCitationNodeConfig,
    ConfigSearchCitationNodeConfig,
    ConfigDataTransformNodeConfig,
    ConfigCreateMessagesNodeConfig,
    ConfigInsertMessagesConfig,
    ConfigRemoveMessageConfig,
    ConfigTokenizerChatTemplateConfig,
    ConfigLlmEngineNodeConfig,
    ConfigResponseParserNodeConfig,
    ConfigProcessingNodeConfig,
    ConfigSqlExecutorNodeConfig,
    ConfigStaticNodeConfig,
    ConfigGenerationNodeConfig,
]


class NodeItem(BaseModel):
    config: Config
    """A data model describing parameters for back-citation using ROUGE similarity.

    metric is the ROUGE metric to use (e.g. rouge1, rouge2, rougeLsum) score is one
    of "precision", "recall", "fmeasure"

    NOTE (john): copied directly from generation.py in order to subclass from
    NodeConfig.
    """

    name: str

    type: str

    inputs: Optional[Dict[str, Union[str, Dict[str, Union[str, object]]]]] = None

    save_to_memory_as: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["UnaryConditionParam"]


class UnaryConditionParam(TypedDict, total=False):
    condition_input_var: Required[str]

    operator: Required[str]

    reference_var: object



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["UnaryCondition"]


class UnaryCondition(BaseModel):
    condition_input_var: str

    operator: str

    reference_var: Optional[object] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .workflow_item import WorkflowItem as WorkflowItem
from .unary_condition import UnaryCondition as UnaryCondition
from .node_item_output import NodeItemOutput as NodeItemOutput
from .config_list_params import ConfigListParams as ConfigListParams
from .plan_config_output import PlanConfigOutput as PlanConfigOutput
from .jinja_node_template import JinjaNodeTemplate as JinjaNodeTemplate
from .workflow_item_param import WorkflowItemParam as WorkflowItemParam
from .config_create_params import ConfigCreateParams as ConfigCreateParams
from .config_list_response import ConfigListResponse as ConfigListResponse
from .config_execute_params import ConfigExecuteParams as ConfigExecuteParams
from .node_item_input_param import NodeItemInputParam as NodeItemInputParam
from .parameter_type_output import ParameterTypeOutput as ParameterTypeOutput
from .unary_condition_param import UnaryConditionParam as UnaryConditionParam
from .config_delete_response import ConfigDeleteResponse as ConfigDeleteResponse
from .workflow_config_output import WorkflowConfigOutput as WorkflowConfigOutput
from .plan_config_input_param import PlanConfigInputParam as PlanConfigInputParam
from .config_retrieve_response import ConfigRetrieveResponse as ConfigRetrieveResponse
from .compound_condition_output import CompoundConditionOutput as CompoundConditionOutput
from .jinja_node_template_param import JinjaNodeTemplateParam as JinjaNodeTemplateParam
from .parameter_type_input_param import ParameterTypeInputParam as ParameterTypeInputParam
from .workflow_config_input_param import WorkflowConfigInputParam as WorkflowConfigInputParam
from .compound_condition_input_param import CompoundConditionInputParam as CompoundConditionInputParam
from .config_retrieve_details_response import ConfigRetrieveDetailsResponse as ConfigRetrieveDetailsResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._compat import PYDANTIC_V2
from .._models import BaseModel
from .workflow_item import WorkflowItem

__all__ = [
    "PlanConfigOutput",
    "Plan",
    "PlanBranchConfigOutput",
    "PlanBranchConfigOutputConditionalWorkflow",
    "PlanLoopConfigOutput",
    "PlanLoopConfigOutputLoopInputs",
    "PlanLoopConfigOutputLoopInputsSelfEdge",
    "Workflows",
    "WorkflowsWorkflowOutput",
]


class PlanBranchConfigOutputConditionalWorkflow(BaseModel):
    condition: Literal["if", "elif", "else"]

    workflow_name: str

    condition_input_var: Optional[str] = None

    condition_tree: Optional["CompoundConditionOutput"] = None
    """Representation of a compound boolean statement, i.e.

    a negation, conjunction, or disjunction of UnaryConditions
    """

    operator: Optional[str] = None

    reference_var: Optional[str] = None

    request_user_input: Optional[Literal["never", "maybe", "always"]] = None
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """

    workflow_alias: Optional[str] = None

    workflow_inputs: Optional[Dict[str, Union[str, Dict[str, Union[str, object]]]]] = None

    workflow_nodes: Optional[List[str]] = None

    workflow_save_outputs: Optional[Dict[str, str]] = None


class PlanBranchConfigOutput(BaseModel):
    branch: str

    conditional_workflows: List[PlanBranchConfigOutputConditionalWorkflow]

    merge_outputs: Optional[Dict[str, List[str]]] = None

    request_user_input: Optional[Literal["never", "maybe", "always"]] = None
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """

    save_merge_to_memory: Optional[Dict[str, str]] = None


class PlanLoopConfigOutputLoopInputsSelfEdge(BaseModel):
    node_name: str

    default_source: Optional[str] = None

    default_value: Optional[object] = None


PlanLoopConfigOutputLoopInputs: TypeAlias = Union[str, PlanLoopConfigOutputLoopInputsSelfEdge]


class PlanLoopConfigOutput(BaseModel):
    condition: "CompoundConditionOutput"
    """Representation of a compound boolean statement, i.e.

    a negation, conjunction, or disjunction of UnaryConditions
    """

    max_iter: int

    name: str

    workflow: WorkflowItem
    """
    Representation of an instance of an abstract Workflow Attributes: workflow_name:
    Key in the map of workflows defined at the top of a plan config workflow_alias:
    Alias of the abstract workflow in the graph. If None, defaults to workflow_name.
    Use in order to re-use the same abstract workflow in multiple portions of the
    graph. workflow_inputs: Inputs to the workflow can be: 1) empty if this workflow
    does not receive input from another workflow, 2) a dictionary mapping another
    workflow's outputs or a branch's merged outputs to this workflows input keys For
    case 2, inputs can be a mapping from: a) str to str b) str to dict(str, str) c)
    str to dict(str, dict(str, str)) workflow_save_outputs: If the following
    parameter is set, the node output can ADDITIONALLY be stored under an alias in
    memory. request_user_input: request_user_input behaves as follows: - When set to
    NEVER, input of this step must be available in memory otherwise exception will
    be raised - When set to NEED_BASED, run will _only_ be interrupted when reaching
    this step in order to request required input fields in case they don't already
    exist in memory from previous node outputs or user inputs - When set to ALWAYS,
    run will _always_ be interrupted when reaching this step in order to request
    these input fields, whether they exist in memory or not
    """

    loop_inputs: Optional[Dict[str, PlanLoopConfigOutputLoopInputs]] = None

    merge_outputs: Optional[Dict[str, str]] = None

    request_user_input: Optional[Literal["never", "maybe", "always"]] = None
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """


Plan: TypeAlias = Union[WorkflowItem, PlanBranchConfigOutput, PlanLoopConfigOutput]


class WorkflowsWorkflowOutput(BaseModel):
    name: str

    nodes: List["NodeItemOutput"]


Workflows: TypeAlias = Union[WorkflowsWorkflowOutput, str, List["NodeItemOutput"]]


class PlanConfigOutput(BaseModel):
    plan: List[Plan]

    id: Optional[str] = None

    account_id: Optional[str] = None

    application_variant_id: Optional[str] = None

    base_url: Optional[str] = None

    concurrency_default: Optional[bool] = None

    datasets: Optional[List[object]] = None

    egp_api_key_override: Optional[str] = None

    egp_ui_evaluation: Optional[object] = None

    evaluations: Optional[List["NodeItemOutput"]] = None

    final_output_nodes: Optional[List[str]] = None

    nodes_to_log: Union[str, List[str], None] = None

    num_workers: Optional[int] = None

    streaming_nodes: Optional[List[str]] = None

    type: Optional[Literal["workflow", "plan", "state_machine"]] = None

    workflows: Optional[Dict[str, Workflows]] = None


from .node_item_output import NodeItemOutput
from .compound_condition_output import CompoundConditionOutput

if PYDANTIC_V2:
    PlanConfigOutput.model_rebuild()
    PlanBranchConfigOutput.model_rebuild()
    PlanBranchConfigOutputConditionalWorkflow.model_rebuild()
    PlanLoopConfigOutput.model_rebuild()
    PlanLoopConfigOutputLoopInputsSelfEdge.model_rebuild()
    WorkflowsWorkflowOutput.model_rebuild()
else:
    PlanConfigOutput.update_forward_refs()  # type: ignore
    PlanBranchConfigOutput.update_forward_refs()  # type: ignore
    PlanBranchConfigOutputConditionalWorkflow.update_forward_refs()  # type: ignore
    PlanLoopConfigOutput.update_forward_refs()  # type: ignore
    PlanLoopConfigOutputLoopInputsSelfEdge.update_forward_refs()  # type: ignore
    WorkflowsWorkflowOutput.update_forward_refs()  # type: ignore



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._compat import PYDANTIC_V2
from .._models import BaseModel
from .unary_condition import UnaryCondition

__all__ = ["CompoundConditionOutput", "Condition"]

Condition: TypeAlias = Union[UnaryCondition, "CompoundConditionOutput"]


class CompoundConditionOutput(BaseModel):
    conditions: Optional[List[Condition]] = None

    input_names: Optional[List[str]] = None

    logical_operator: Optional[Literal["ALL", "ANY", "NOT"]] = None


if PYDANTIC_V2:
    CompoundConditionOutput.model_rebuild()
else:
    CompoundConditionOutput.update_forward_refs()  # type: ignore



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .._compat import PYDANTIC_V2
from .._models import BaseModel

__all__ = ["ParameterTypeOutput", "AdditionalProperties", "AnyOf", "Items"]

AdditionalProperties: TypeAlias = Union[
    Literal["object", "array"], Literal["string", "number", "integer", "boolean", "null"], "ParameterTypeOutput", None
]

AnyOf: TypeAlias = Union[
    Literal["object", "array"], Literal["string", "number", "integer", "boolean", "null"], "ParameterTypeOutput"
]

Items: TypeAlias = Union[
    Literal["object", "array"], Literal["string", "number", "integer", "boolean", "null"], "ParameterTypeOutput", None
]


class ParameterTypeOutput(BaseModel):
    additional_properties: Optional[AdditionalProperties] = FieldInfo(alias="additionalProperties", default=None)

    any_of: Optional[List[AnyOf]] = FieldInfo(alias="anyOf", default=None)

    items: Optional[Items] = None

    property_names: Union[
        Literal["string", "number", "integer", "boolean", "null"],
        List[Literal["string", "number", "integer", "boolean", "null"]],
        None,
    ] = FieldInfo(alias="propertyNames", default=None)

    type: Union[
        Literal["object", "array"],
        Literal["string", "number", "integer", "boolean", "null"],
        List[Literal["object", "array", "string", "number", "integer", "boolean", "null"]],
        None,
    ] = None


if PYDANTIC_V2:
    ParameterTypeOutput.model_rebuild()
else:
    ParameterTypeOutput.update_forward_refs()  # type: ignore



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["WorkflowItemParam"]


class WorkflowItemParam(TypedDict, total=False):
    workflow_name: Required[str]

    request_user_input: Literal["never", "maybe", "always"]
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """

    workflow_alias: Optional[str]

    workflow_inputs: Optional[Dict[str, Union[str, Dict[str, Union[str, object]]]]]

    workflow_save_outputs: Optional[Dict[str, str]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._compat import PYDANTIC_V2
from .._models import BaseModel
from .jinja_node_template import JinjaNodeTemplate

__all__ = [
    "NodeItemOutput",
    "Config",
    "ConfigNodeConfigOutput",
    "ConfigJinjaNodeConfigOutput",
    "ConfigChunkEvaluationNodeConfigOutput",
    "ConfigRerankerNodeConfigOutput",
    "ConfigRetrieverNodeConfigOutput",
    "ConfigCitationNodeConfigOutput",
    "ConfigCitationNodeConfigOutputCitationContext",
    "ConfigSearchCitationNodeConfigOutput",
    "ConfigDataTransformNodeConfigOutput",
    "ConfigCreateMessagesNodeConfigOutput",
    "ConfigCreateMessagesNodeConfigOutputMessageConfig",
    "ConfigCreateMessagesNodeConfigOutputMessageConfigAlternatingRoleMessages",
    "ConfigCreateMessagesNodeConfigOutputMessageConfigSingleRoleMessages",
    "ConfigInsertMessagesConfigOutput",
    "ConfigRemoveMessageConfigOutput",
    "ConfigTokenizerChatTemplateConfigOutput",
    "ConfigLlmEngineNodeConfigOutput",
    "ConfigLlmEngineNodeConfigOutputBatchSysKwargs",
    "ConfigResponseParserNodeConfigOutput",
    "ConfigProcessingNodeConfigOutput",
    "ConfigProcessingNodeConfigOutputFunctionSpecs",
    "ConfigSqlExecutorNodeConfigOutput",
    "ConfigStaticNodeConfigOutput",
    "ConfigGenerationNodeConfigOutput",
]


class ConfigNodeConfigOutput(BaseModel):
    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigJinjaNodeConfigOutput(BaseModel):
    context_chunks_key: Optional[str] = None

    data_transformations: Optional[Dict[str, JinjaNodeTemplate]] = None

    llm_model: Optional[str] = None

    log_output: Optional[bool] = None

    log_prefix: Optional[str] = None

    max_tokens: Optional[int] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    output_template: Optional[JinjaNodeTemplate] = None
    """
    Base model for a Jinja template. Guaranteed to store a string that can be read
    in to Template().
    """

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None

    verbose: Optional[bool] = None


class ConfigChunkEvaluationNodeConfigOutput(BaseModel):
    top_k_thresholds: List[int]

    fuzzy_match_threshold: Optional[float] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    require_all: Optional[bool] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigRerankerNodeConfigOutput(BaseModel):
    num_to_return: int

    scorers: List[object]

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    score_threshold: Optional[float] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigRetrieverNodeConfigOutput(BaseModel):
    num_to_return: int

    exact_knn_search: Optional[bool] = None

    knowledge_base_id: Optional[str] = None

    knowledge_base_name: Optional[str] = None

    metadata: Optional[Dict[str, Optional[str]]] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigCitationNodeConfigOutputCitationContext(BaseModel):
    generate_with_llm: Optional[bool] = None

    metric: Optional[str] = None

    min_similarity: Optional[float] = None

    score: Optional[Literal["precision", "recall", "fmeasure"]] = None


class ConfigCitationNodeConfigOutput(BaseModel):
    citation_type: Literal["rouge", "model_defined"]

    citation_context: Optional[ConfigCitationNodeConfigOutputCitationContext] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    s3_path_override: Optional[str] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigSearchCitationNodeConfigOutput(BaseModel):
    end_search_regex: str

    search_regex: str

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigDataTransformNodeConfigOutput(BaseModel):
    action: str

    additional_inputs: Optional[object] = None

    apply_to_dictlist_leaves: Optional[bool] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigCreateMessagesNodeConfigOutputMessageConfigAlternatingRoleMessages(BaseModel):
    role_value_pairs: List[Dict[str, str]]


class ConfigCreateMessagesNodeConfigOutputMessageConfigSingleRoleMessages(BaseModel):
    content: str

    role: str


ConfigCreateMessagesNodeConfigOutputMessageConfig: TypeAlias = Union[
    ConfigCreateMessagesNodeConfigOutputMessageConfigAlternatingRoleMessages,
    ConfigCreateMessagesNodeConfigOutputMessageConfigSingleRoleMessages,
]


class ConfigCreateMessagesNodeConfigOutput(BaseModel):
    message_configs: List[ConfigCreateMessagesNodeConfigOutputMessageConfig]

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigInsertMessagesConfigOutput(BaseModel):
    index: int

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigRemoveMessageConfigOutput(BaseModel):
    index: int

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigTokenizerChatTemplateConfigOutput(BaseModel):
    llm_model: str

    add_generation_prompt: Optional[bool] = None

    kwargs: Optional[object] = None

    max_length: Optional[int] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    padding: Optional[bool] = None

    truncation: Optional[bool] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigLlmEngineNodeConfigOutputBatchSysKwargs(BaseModel):
    checkpoint_path: Optional[str] = None

    labels: Optional[Dict[str, str]] = None

    num_shards: Optional[int] = None

    seed: Optional[int] = None


class ConfigLlmEngineNodeConfigOutput(BaseModel):
    llm_model: str

    batch_run_mode: Optional[Literal["sync", "async"]] = None

    batch_sys_kwargs: Optional[ConfigLlmEngineNodeConfigOutputBatchSysKwargs] = None

    frequency_penalty: Optional[float] = None

    guided_choice: Optional[List[str]] = None

    guided_json: Optional[object] = None

    guided_regex: Optional[str] = None

    include_stop_str_in_output: Optional[bool] = None

    max_tokens: Optional[int] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    presence_penalty: Optional[float] = None

    stop_sequences: Optional[List[str]] = None

    temperature: Optional[float] = None

    timeout: Optional[int] = None

    top_k: Optional[int] = None

    top_p: Optional[float] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigResponseParserNodeConfigOutput(BaseModel):
    action: str

    reference_value: object

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigProcessingNodeConfigOutputFunctionSpecs(BaseModel):
    kwargs: object

    path: str


class ConfigProcessingNodeConfigOutput(BaseModel):
    function_specs: Dict[str, ConfigProcessingNodeConfigOutputFunctionSpecs]

    return_key: str

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigSqlExecutorNodeConfigOutput(BaseModel):
    connector_kwargs: Dict[str, str]

    connector_type: Optional[Literal["snowflake"]] = None

    log_queries: Optional[bool] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    return_type: Optional[Literal["df", "dicts", "markdown", "json", "str"]] = None

    schema_remapping_file: Optional[str] = None

    secrets: Optional[List[str]] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


class ConfigStaticNodeConfigOutput(BaseModel):
    from_file: Union[List[object], str, object, None] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None

    value: Optional[object] = None


class ConfigGenerationNodeConfigOutput(BaseModel):
    llm_model: Optional[str] = None

    llm_model_deployment: Optional[str] = None

    llm_model_instance: Optional[str] = None

    max_tokens: Optional[int] = None

    node_metadata: Optional[List[str]] = None

    num_workers: Optional[int] = None

    stop_sequences: Optional[List[str]] = None

    strip_whitespace: Optional[bool] = None

    temperature: Optional[float] = None

    tool_name: Optional[str] = None

    type_hints: Optional[Dict[str, "ParameterTypeOutput"]] = None


Config: TypeAlias = Union[
    ConfigNodeConfigOutput,
    ConfigJinjaNodeConfigOutput,
    ConfigChunkEvaluationNodeConfigOutput,
    ConfigRerankerNodeConfigOutput,
    ConfigRetrieverNodeConfigOutput,
    ConfigCitationNodeConfigOutput,
    ConfigSearchCitationNodeConfigOutput,
    ConfigDataTransformNodeConfigOutput,
    ConfigCreateMessagesNodeConfigOutput,
    ConfigInsertMessagesConfigOutput,
    ConfigRemoveMessageConfigOutput,
    ConfigTokenizerChatTemplateConfigOutput,
    ConfigLlmEngineNodeConfigOutput,
    ConfigResponseParserNodeConfigOutput,
    ConfigProcessingNodeConfigOutput,
    ConfigSqlExecutorNodeConfigOutput,
    ConfigStaticNodeConfigOutput,
    ConfigGenerationNodeConfigOutput,
]


class NodeItemOutput(BaseModel):
    config: Config
    """A data model describing parameters for back-citation using ROUGE similarity.

    metric is the ROUGE metric to use (e.g. rouge1, rouge2, rougeLsum) score is one
    of "precision", "recall", "fmeasure"

    NOTE (john): copied directly from generation.py in order to subclass from
    NodeConfig.
    """

    name: str

    type: str

    inputs: Optional[Dict[str, Union[str, Dict[str, Union[str, object]]]]] = None

    save_to_memory_as: Optional[str] = None


from .parameter_type_output import ParameterTypeOutput

if PYDANTIC_V2:
    NodeItemOutput.model_rebuild()
    ConfigNodeConfigOutput.model_rebuild()
    ConfigJinjaNodeConfigOutput.model_rebuild()
    ConfigChunkEvaluationNodeConfigOutput.model_rebuild()
    ConfigRerankerNodeConfigOutput.model_rebuild()
    ConfigRetrieverNodeConfigOutput.model_rebuild()
    ConfigCitationNodeConfigOutput.model_rebuild()
    ConfigCitationNodeConfigOutputCitationContext.model_rebuild()
    ConfigSearchCitationNodeConfigOutput.model_rebuild()
    ConfigDataTransformNodeConfigOutput.model_rebuild()
    ConfigCreateMessagesNodeConfigOutput.model_rebuild()
    ConfigCreateMessagesNodeConfigOutputMessageConfigAlternatingRoleMessages.model_rebuild()
    ConfigCreateMessagesNodeConfigOutputMessageConfigSingleRoleMessages.model_rebuild()
    ConfigInsertMessagesConfigOutput.model_rebuild()
    ConfigRemoveMessageConfigOutput.model_rebuild()
    ConfigTokenizerChatTemplateConfigOutput.model_rebuild()
    ConfigLlmEngineNodeConfigOutput.model_rebuild()
    ConfigLlmEngineNodeConfigOutputBatchSysKwargs.model_rebuild()
    ConfigResponseParserNodeConfigOutput.model_rebuild()
    ConfigProcessingNodeConfigOutput.model_rebuild()
    ConfigProcessingNodeConfigOutputFunctionSpecs.model_rebuild()
    ConfigSqlExecutorNodeConfigOutput.model_rebuild()
    ConfigStaticNodeConfigOutput.model_rebuild()
    ConfigGenerationNodeConfigOutput.model_rebuild()
else:
    NodeItemOutput.update_forward_refs()  # type: ignore
    ConfigNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigJinjaNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigChunkEvaluationNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigRerankerNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigRetrieverNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigCitationNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigCitationNodeConfigOutputCitationContext.update_forward_refs()  # type: ignore
    ConfigSearchCitationNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigDataTransformNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigCreateMessagesNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigCreateMessagesNodeConfigOutputMessageConfigAlternatingRoleMessages.update_forward_refs()  # type: ignore
    ConfigCreateMessagesNodeConfigOutputMessageConfigSingleRoleMessages.update_forward_refs()  # type: ignore
    ConfigInsertMessagesConfigOutput.update_forward_refs()  # type: ignore
    ConfigRemoveMessageConfigOutput.update_forward_refs()  # type: ignore
    ConfigTokenizerChatTemplateConfigOutput.update_forward_refs()  # type: ignore
    ConfigLlmEngineNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigLlmEngineNodeConfigOutputBatchSysKwargs.update_forward_refs()  # type: ignore
    ConfigResponseParserNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigProcessingNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigProcessingNodeConfigOutputFunctionSpecs.update_forward_refs()  # type: ignore
    ConfigSqlExecutorNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigStaticNodeConfigOutput.update_forward_refs()  # type: ignore
    ConfigGenerationNodeConfigOutput.update_forward_refs()  # type: ignore



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .health import (
    HealthResource,
    AsyncHealthResource,
    HealthResourceWithRawResponse,
    AsyncHealthResourceWithRawResponse,
    HealthResourceWithStreamingResponse,
    AsyncHealthResourceWithStreamingResponse,
)
from .configs import (
    ConfigsResource,
    AsyncConfigsResource,
    ConfigsResourceWithRawResponse,
    AsyncConfigsResourceWithRawResponse,
    ConfigsResourceWithStreamingResponse,
    AsyncConfigsResourceWithStreamingResponse,
)
from .healthz import (
    HealthzResource,
    AsyncHealthzResource,
    HealthzResourceWithRawResponse,
    AsyncHealthzResourceWithRawResponse,
    HealthzResourceWithStreamingResponse,
    AsyncHealthzResourceWithStreamingResponse,
)

__all__ = [
    "HealthResource",
    "AsyncHealthResource",
    "HealthResourceWithRawResponse",
    "AsyncHealthResourceWithRawResponse",
    "HealthResourceWithStreamingResponse",
    "AsyncHealthResourceWithStreamingResponse",
    "HealthzResource",
    "AsyncHealthzResource",
    "HealthzResourceWithRawResponse",
    "AsyncHealthzResourceWithRawResponse",
    "HealthzResourceWithStreamingResponse",
    "AsyncHealthzResourceWithStreamingResponse",
    "ConfigsResource",
    "AsyncConfigsResource",
    "ConfigsResourceWithRawResponse",
    "AsyncConfigsResourceWithRawResponse",
    "ConfigsResourceWithStreamingResponse",
    "AsyncConfigsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .workflow_item_param import WorkflowItemParam
from .node_item_input_param import NodeItemInputParam
from .compound_condition_input_param import CompoundConditionInputParam

__all__ = [
    "PlanConfigInputParam",
    "Plan",
    "PlanBranchConfigInput",
    "PlanBranchConfigInputConditionalWorkflow",
    "PlanLoopConfigInput",
    "PlanLoopConfigInputLoopInputs",
    "PlanLoopConfigInputLoopInputsSelfEdge",
    "Workflows",
    "WorkflowsWorkflowInput",
]


class PlanBranchConfigInputConditionalWorkflow(TypedDict, total=False):
    condition: Required[Literal["if", "elif", "else"]]

    workflow_name: Required[str]

    condition_input_var: Optional[str]

    condition_tree: CompoundConditionInputParam
    """Representation of a compound boolean statement, i.e.

    a negation, conjunction, or disjunction of UnaryConditions
    """

    operator: Optional[str]

    reference_var: Optional[str]

    request_user_input: Literal["never", "maybe", "always"]
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """

    workflow_alias: Optional[str]

    workflow_inputs: Optional[Dict[str, Union[str, Dict[str, Union[str, object]]]]]

    workflow_nodes: List[str]

    workflow_save_outputs: Optional[Dict[str, str]]


class PlanBranchConfigInput(TypedDict, total=False):
    branch: Required[str]

    conditional_workflows: Required[Iterable[PlanBranchConfigInputConditionalWorkflow]]

    merge_outputs: Dict[str, List[str]]

    request_user_input: Literal["never", "maybe", "always"]
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """

    save_merge_to_memory: Dict[str, str]


class PlanLoopConfigInputLoopInputsSelfEdge(TypedDict, total=False):
    node_name: Required[str]

    default_source: Optional[str]

    default_value: Optional[object]


PlanLoopConfigInputLoopInputs: TypeAlias = Union[str, PlanLoopConfigInputLoopInputsSelfEdge]


class PlanLoopConfigInput(TypedDict, total=False):
    condition: Required[CompoundConditionInputParam]
    """Representation of a compound boolean statement, i.e.

    a negation, conjunction, or disjunction of UnaryConditions
    """

    max_iter: Required[int]

    name: Required[str]

    workflow: Required[WorkflowItemParam]
    """
    Representation of an instance of an abstract Workflow Attributes: workflow_name:
    Key in the map of workflows defined at the top of a plan config workflow_alias:
    Alias of the abstract workflow in the graph. If None, defaults to workflow_name.
    Use in order to re-use the same abstract workflow in multiple portions of the
    graph. workflow_inputs: Inputs to the workflow can be: 1) empty if this workflow
    does not receive input from another workflow, 2) a dictionary mapping another
    workflow's outputs or a branch's merged outputs to this workflows input keys For
    case 2, inputs can be a mapping from: a) str to str b) str to dict(str, str) c)
    str to dict(str, dict(str, str)) workflow_save_outputs: If the following
    parameter is set, the node output can ADDITIONALLY be stored under an alias in
    memory. request_user_input: request_user_input behaves as follows: - When set to
    NEVER, input of this step must be available in memory otherwise exception will
    be raised - When set to NEED_BASED, run will _only_ be interrupted when reaching
    this step in order to request required input fields in case they don't already
    exist in memory from previous node outputs or user inputs - When set to ALWAYS,
    run will _always_ be interrupted when reaching this step in order to request
    these input fields, whether they exist in memory or not
    """

    loop_inputs: Dict[str, PlanLoopConfigInputLoopInputs]

    merge_outputs: Dict[str, str]

    request_user_input: Literal["never", "maybe", "always"]
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """


Plan: TypeAlias = Union[WorkflowItemParam, PlanBranchConfigInput, PlanLoopConfigInput]


class WorkflowsWorkflowInput(TypedDict, total=False):
    name: Required[str]

    nodes: Required[Iterable[NodeItemInputParam]]


Workflows: TypeAlias = Union[WorkflowsWorkflowInput, str, Iterable[NodeItemInputParam]]


class PlanConfigInputParam(TypedDict, total=False):
    plan: Required[Iterable[Plan]]

    id: Optional[str]

    account_id: Optional[str]

    application_variant_id: Optional[str]

    base_url: Optional[str]

    concurrency_default: Optional[bool]

    datasets: Optional[Iterable[object]]

    egp_api_key_override: Optional[str]

    egp_ui_evaluation: Optional[object]

    evaluations: Optional[Iterable[NodeItemInputParam]]

    final_output_nodes: Optional[List[str]]

    nodes_to_log: Union[str, List[str], None]

    num_workers: Optional[int]

    streaming_nodes: Optional[List[str]]

    type: Optional[Literal["workflow", "plan", "state_machine"]]

    workflows: Dict[str, Workflows]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .node_item import NodeItem

__all__ = ["WorkflowConfig"]


class WorkflowConfig(BaseModel):
    workflow: List[NodeItem]

    id: Optional[str] = None

    account_id: Optional[str] = None

    application_variant_id: Optional[str] = None

    base_url: Optional[str] = None

    concurrency_default: Optional[bool] = None

    datasets: Optional[List[object]] = None

    egp_api_key_override: Optional[str] = None

    egp_ui_evaluation: Optional[object] = None

    evaluations: Optional[List[NodeItem]] = None

    final_output_nodes: Optional[List[str]] = None

    nodes_to_log: Union[str, List[str], None] = None

    num_workers: Optional[int] = None

    streaming_nodes: Optional[List[str]] = None

    type: Optional[Literal["workflow", "plan", "state_machine"]] = None
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from typing_extensions import Literal, Required, TypedDict

from .node_item_param import NodeItemParam

__all__ = ["WorkflowConfigParam"]


class WorkflowConfigParam(TypedDict, total=False):
    workflow: Required[Iterable[NodeItemParam]]

    id: str

    account_id: str

    application_variant_id: str

    base_url: str

    concurrency_default: bool

    datasets: Iterable[object]

    egp_api_key_override: str

    egp_ui_evaluation: object

    evaluations: Iterable[NodeItemParam]

    final_output_nodes: List[str]

    nodes_to_log: Union[str, List[str]]

    num_workers: int

    streaming_nodes: List[str]

    type: Literal["workflow", "plan", "state_machine"]
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .jinja_node_template_param import JinjaNodeTemplateParam
from .parameter_type_input_param import ParameterTypeInputParam

__all__ = [
    "NodeItemInputParam",
    "Config",
    "ConfigNodeConfigInput",
    "ConfigJinjaNodeConfigInput",
    "ConfigChunkEvaluationNodeConfigInput",
    "ConfigRerankerNodeConfigInput",
    "ConfigRetrieverNodeConfigInput",
    "ConfigCitationNodeConfigInput",
    "ConfigCitationNodeConfigInputCitationContext",
    "ConfigSearchCitationNodeConfigInput",
    "ConfigDataTransformNodeConfigInput",
    "ConfigCreateMessagesNodeConfigInput",
    "ConfigCreateMessagesNodeConfigInputMessageConfig",
    "ConfigCreateMessagesNodeConfigInputMessageConfigAlternatingRoleMessages",
    "ConfigCreateMessagesNodeConfigInputMessageConfigSingleRoleMessages",
    "ConfigInsertMessagesConfigInput",
    "ConfigRemoveMessageConfigInput",
    "ConfigTokenizerChatTemplateConfigInput",
    "ConfigLlmEngineNodeConfigInput",
    "ConfigLlmEngineNodeConfigInputBatchSysKwargs",
    "ConfigResponseParserNodeConfigInput",
    "ConfigProcessingNodeConfigInput",
    "ConfigProcessingNodeConfigInputFunctionSpecs",
    "ConfigSqlExecutorNodeConfigInput",
    "ConfigStaticNodeConfigInput",
    "ConfigGenerationNodeConfigInput",
]


class ConfigNodeConfigInput(TypedDict, total=False):
    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigJinjaNodeConfigInput(TypedDict, total=False):
    context_chunks_key: Optional[str]

    data_transformations: Dict[str, JinjaNodeTemplateParam]

    llm_model: Optional[str]

    log_output: bool

    log_prefix: str

    max_tokens: Optional[int]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    output_template: JinjaNodeTemplateParam
    """
    Base model for a Jinja template. Guaranteed to store a string that can be read
    in to Template().
    """

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]

    verbose: bool


class ConfigChunkEvaluationNodeConfigInput(TypedDict, total=False):
    top_k_thresholds: Required[Iterable[int]]

    fuzzy_match_threshold: float

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    require_all: bool

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigRerankerNodeConfigInput(TypedDict, total=False):
    num_to_return: Required[int]

    scorers: Required[Iterable[object]]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    score_threshold: Optional[float]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigRetrieverNodeConfigInput(TypedDict, total=False):
    num_to_return: Required[int]

    exact_knn_search: Optional[bool]

    knowledge_base_id: Optional[str]

    knowledge_base_name: Optional[str]

    metadata: Optional[Dict[str, Optional[str]]]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigCitationNodeConfigInputCitationContext(TypedDict, total=False):
    generate_with_llm: bool

    metric: Optional[str]

    min_similarity: Optional[float]

    score: Optional[Literal["precision", "recall", "fmeasure"]]


class ConfigCitationNodeConfigInput(TypedDict, total=False):
    citation_type: Required[Literal["rouge", "model_defined"]]

    citation_context: ConfigCitationNodeConfigInputCitationContext

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    s3_path_override: Optional[str]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigSearchCitationNodeConfigInput(TypedDict, total=False):
    end_search_regex: Required[str]

    search_regex: Required[str]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigDataTransformNodeConfigInput(TypedDict, total=False):
    action: Required[str]

    additional_inputs: object

    apply_to_dictlist_leaves: bool

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigCreateMessagesNodeConfigInputMessageConfigAlternatingRoleMessages(TypedDict, total=False):
    role_value_pairs: Required[Iterable[Dict[str, str]]]


class ConfigCreateMessagesNodeConfigInputMessageConfigSingleRoleMessages(TypedDict, total=False):
    content: Required[str]

    role: Required[str]


ConfigCreateMessagesNodeConfigInputMessageConfig: TypeAlias = Union[
    ConfigCreateMessagesNodeConfigInputMessageConfigAlternatingRoleMessages,
    ConfigCreateMessagesNodeConfigInputMessageConfigSingleRoleMessages,
]


class ConfigCreateMessagesNodeConfigInput(TypedDict, total=False):
    message_configs: Required[Iterable[ConfigCreateMessagesNodeConfigInputMessageConfig]]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigInsertMessagesConfigInput(TypedDict, total=False):
    index: Required[int]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigRemoveMessageConfigInput(TypedDict, total=False):
    index: Required[int]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigTokenizerChatTemplateConfigInput(TypedDict, total=False):
    llm_model: Required[str]

    add_generation_prompt: bool

    kwargs: object

    max_length: Optional[int]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    padding: bool

    truncation: bool

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigLlmEngineNodeConfigInputBatchSysKwargs(TypedDict, total=False):
    checkpoint_path: Optional[str]

    labels: Optional[Dict[str, str]]

    num_shards: Optional[int]

    seed: Optional[int]


class ConfigLlmEngineNodeConfigInput(TypedDict, total=False):
    llm_model: Required[str]

    batch_run_mode: Literal["sync", "async"]

    batch_sys_kwargs: ConfigLlmEngineNodeConfigInputBatchSysKwargs

    frequency_penalty: Optional[float]

    guided_choice: Optional[List[str]]

    guided_json: Optional[object]

    guided_regex: Optional[str]

    include_stop_str_in_output: Optional[bool]

    max_tokens: Optional[int]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    presence_penalty: Optional[float]

    stop_sequences: Optional[List[str]]

    temperature: Optional[float]

    timeout: int

    top_k: Optional[int]

    top_p: Optional[float]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigResponseParserNodeConfigInput(TypedDict, total=False):
    action: Required[str]

    reference_value: Required[object]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigProcessingNodeConfigInputFunctionSpecs(TypedDict, total=False):
    kwargs: Required[object]

    path: Required[str]


class ConfigProcessingNodeConfigInput(TypedDict, total=False):
    function_specs: Required[Dict[str, ConfigProcessingNodeConfigInputFunctionSpecs]]

    return_key: Required[str]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigSqlExecutorNodeConfigInput(TypedDict, total=False):
    connector_kwargs: Required[Dict[str, str]]

    connector_type: Literal["snowflake"]

    log_queries: bool

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    return_type: Literal["df", "dicts", "markdown", "json", "str"]

    schema_remapping_file: Optional[str]

    secrets: List[str]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


class ConfigStaticNodeConfigInput(TypedDict, total=False):
    from_file: Union[Iterable[object], str, object, None]

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]

    value: Optional[object]


class ConfigGenerationNodeConfigInput(TypedDict, total=False):
    llm_model: str

    llm_model_deployment: str

    llm_model_instance: str

    max_tokens: int

    node_metadata: Optional[List[str]]

    num_workers: Optional[int]

    stop_sequences: Optional[List[str]]

    strip_whitespace: bool

    temperature: float

    tool_name: Optional[str]

    type_hints: Optional[Dict[str, ParameterTypeInputParam]]


Config: TypeAlias = Union[
    ConfigNodeConfigInput,
    ConfigJinjaNodeConfigInput,
    ConfigChunkEvaluationNodeConfigInput,
    ConfigRerankerNodeConfigInput,
    ConfigRetrieverNodeConfigInput,
    ConfigCitationNodeConfigInput,
    ConfigSearchCitationNodeConfigInput,
    ConfigDataTransformNodeConfigInput,
    ConfigCreateMessagesNodeConfigInput,
    ConfigInsertMessagesConfigInput,
    ConfigRemoveMessageConfigInput,
    ConfigTokenizerChatTemplateConfigInput,
    ConfigLlmEngineNodeConfigInput,
    ConfigResponseParserNodeConfigInput,
    ConfigProcessingNodeConfigInput,
    ConfigSqlExecutorNodeConfigInput,
    ConfigStaticNodeConfigInput,
    ConfigGenerationNodeConfigInput,
]


class NodeItemInputParam(TypedDict, total=False):
    config: Required[Config]
    """A data model describing parameters for back-citation using ROUGE similarity.

    metric is the ROUGE metric to use (e.g. rouge1, rouge2, rougeLsum) score is one
    of "precision", "recall", "fmeasure"

    NOTE (john): copied directly from generation.py in order to subclass from
    NodeConfig.
    """

    name: Required[str]

    type: Required[str]

    inputs: Dict[str, Union[str, Dict[str, Union[str, object]]]]

    save_to_memory_as: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ConfigListParams"]


class ConfigListParams(TypedDict, total=False):
    config_type: Required[Literal["workflow", "plan", "state_machine"]]

    account_id: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import TypedDict

__all__ = ["JinjaNodeTemplateParam"]


class JinjaNodeTemplateParam(TypedDict, total=False):
    jinja_helper_functions: Optional[List[Union[str, object]]]

    jinja_template_path: Optional[str]

    jinja_template_str: Optional[str]
    """Raw template to apply to the data.

    This should be a Jinja2 template string. Please note, the data will be mapped as
    'value' in the template. Default None corresponds to {{value}}. Should access
    property `jinja_template_str` or field `jinja_template_str_loaded` for the
    loaded template data
    """

    jinja_template_str_loaded: Optional[str]
    """
    The original jinja_template_str field from the config might not contain the
    needed template, and we may need to load S3 data specified with
    `jinja_template_path`. This field caches the loaded template content, it is also
    accessed through property `jinja_template_str`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import Literal

from .._compat import PYDANTIC_V2
from .._models import BaseModel

__all__ = ["WorkflowConfigOutput"]


class WorkflowConfigOutput(BaseModel):
    workflow: List["NodeItemOutput"]

    id: Optional[str] = None

    account_id: Optional[str] = None

    application_variant_id: Optional[str] = None

    base_url: Optional[str] = None

    concurrency_default: Optional[bool] = None

    datasets: Optional[List[object]] = None

    egp_api_key_override: Optional[str] = None

    egp_ui_evaluation: Optional[object] = None

    evaluations: Optional[List["NodeItemOutput"]] = None

    final_output_nodes: Optional[List[str]] = None

    nodes_to_log: Union[str, List[str], None] = None

    num_workers: Optional[int] = None

    streaming_nodes: Optional[List[str]] = None

    type: Optional[Literal["workflow", "plan", "state_machine"]] = None


from .node_item_output import NodeItemOutput

if PYDANTIC_V2:
    WorkflowConfigOutput.model_rebuild()
else:
    WorkflowConfigOutput.update_forward_refs()  # type: ignore



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from .node_item_input_param import NodeItemInputParam

__all__ = ["WorkflowConfigInputParam"]


class WorkflowConfigInputParam(TypedDict, total=False):
    workflow: Required[Iterable[NodeItemInputParam]]

    id: Optional[str]

    account_id: Optional[str]

    application_variant_id: Optional[str]

    base_url: Optional[str]

    concurrency_default: Optional[bool]

    datasets: Optional[Iterable[object]]

    egp_api_key_override: Optional[str]

    egp_ui_evaluation: Optional[object]

    evaluations: Optional[Iterable[NodeItemInputParam]]

    final_output_nodes: Optional[List[str]]

    nodes_to_log: Union[str, List[str], None]

    num_workers: Optional[int]

    streaming_nodes: Optional[List[str]]

    type: Optional[Literal["workflow", "plan", "state_machine"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from ..types import config_list_params, config_create_params, config_execute_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.config_list_response import ConfigListResponse
from ..types.config_delete_response import ConfigDeleteResponse
from ..types.config_retrieve_response import ConfigRetrieveResponse
from ..types.config_retrieve_details_response import ConfigRetrieveDetailsResponse

__all__ = ["ConfigsResource", "AsyncConfigsResource"]


class ConfigsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/agents-python#accessing-raw-response-data-eg-headers
        """
        return ConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/agents-python#with_streaming_response
        """
        return ConfigsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        config: config_create_params.Config,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Create Config

        Args:
          config: Representation of a plan, i.e.

        a composition of workflows and branch complexes

              Public attributes: workflows: maps a workflow "name" to either a workflow yaml
              file or an inline workflow definition plan: representation of the graph
              connecting workflows / branch complexes

              Private attributes: helper_workflows: a list of workflows created by default, in
              order to support loops and other control flow features

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/configs/create",
            body=maybe_transform({"config": config}, config_create_params.ConfigCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def retrieve(
        self,
        config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigRetrieveResponse:
        """
        Get Config By Id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_id:
            raise ValueError(f"Expected a non-empty value for `config_id` but received {config_id!r}")
        return cast(
            ConfigRetrieveResponse,
            self._get(
                f"/v1/configs/{config_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ConfigRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        config_type: Literal["workflow", "plan", "state_machine"],
        account_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigListResponse:
        """
        List Configs

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/configs/list",
            body=maybe_transform(
                {
                    "config_type": config_type,
                    "account_id": account_id,
                },
                config_list_params.ConfigListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigListResponse,
        )

    def delete(
        self,
        config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigDeleteResponse:
        """
        Delete Config By Id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_id:
            raise ValueError(f"Expected a non-empty value for `config_id` but received {config_id!r}")
        return self._delete(
            f"/v1/configs/{config_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigDeleteResponse,
        )

    def execute(
        self,
        config_id: str,
        *,
        messages: Optional[Iterable[config_execute_params.Message]],
        metadata: Optional[object],
        run_id: Optional[str],
        session_id: str,
        id: str | NotGiven = NOT_GIVEN,
        concurrent: Optional[bool] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Execute Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_id:
            raise ValueError(f"Expected a non-empty value for `config_id` but received {config_id!r}")
        return self._post(
            f"/v1/configs/execute/{config_id}",
            body=maybe_transform(
                {
                    "messages": messages,
                    "metadata": metadata,
                    "run_id": run_id,
                    "session_id": session_id,
                    "id": id,
                    "concurrent": concurrent,
                    "stream": stream,
                },
                config_execute_params.ConfigExecuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def retrieve_details(
        self,
        config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigRetrieveDetailsResponse:
        """
        Get details about the building blocks of the config, like its nodes, edges
        between nodes, and user inputs

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_id:
            raise ValueError(f"Expected a non-empty value for `config_id` but received {config_id!r}")
        return self._get(
            f"/v1/configs/{config_id}/details",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigRetrieveDetailsResponse,
        )


class AsyncConfigsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/agents-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/agents-python#with_streaming_response
        """
        return AsyncConfigsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        config: config_create_params.Config,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """Create Config

        Args:
          config: Representation of a plan, i.e.

        a composition of workflows and branch complexes

              Public attributes: workflows: maps a workflow "name" to either a workflow yaml
              file or an inline workflow definition plan: representation of the graph
              connecting workflows / branch complexes

              Private attributes: helper_workflows: a list of workflows created by default, in
              order to support loops and other control flow features

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/configs/create",
            body=await async_maybe_transform({"config": config}, config_create_params.ConfigCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def retrieve(
        self,
        config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigRetrieveResponse:
        """
        Get Config By Id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_id:
            raise ValueError(f"Expected a non-empty value for `config_id` but received {config_id!r}")
        return cast(
            ConfigRetrieveResponse,
            await self._get(
                f"/v1/configs/{config_id}",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ConfigRetrieveResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def list(
        self,
        *,
        config_type: Literal["workflow", "plan", "state_machine"],
        account_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigListResponse:
        """
        List Configs

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/configs/list",
            body=await async_maybe_transform(
                {
                    "config_type": config_type,
                    "account_id": account_id,
                },
                config_list_params.ConfigListParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigListResponse,
        )

    async def delete(
        self,
        config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigDeleteResponse:
        """
        Delete Config By Id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_id:
            raise ValueError(f"Expected a non-empty value for `config_id` but received {config_id!r}")
        return await self._delete(
            f"/v1/configs/{config_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigDeleteResponse,
        )

    async def execute(
        self,
        config_id: str,
        *,
        messages: Optional[Iterable[config_execute_params.Message]],
        metadata: Optional[object],
        run_id: Optional[str],
        session_id: str,
        id: str | NotGiven = NOT_GIVEN,
        concurrent: Optional[bool] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        """
        Execute Config

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_id:
            raise ValueError(f"Expected a non-empty value for `config_id` but received {config_id!r}")
        return await self._post(
            f"/v1/configs/execute/{config_id}",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "metadata": metadata,
                    "run_id": run_id,
                    "session_id": session_id,
                    "id": id,
                    "concurrent": concurrent,
                    "stream": stream,
                },
                config_execute_params.ConfigExecuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def retrieve_details(
        self,
        config_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConfigRetrieveDetailsResponse:
        """
        Get details about the building blocks of the config, like its nodes, edges
        between nodes, and user inputs

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not config_id:
            raise ValueError(f"Expected a non-empty value for `config_id` but received {config_id!r}")
        return await self._get(
            f"/v1/configs/{config_id}/details",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConfigRetrieveDetailsResponse,
        )


class ConfigsResourceWithRawResponse:
    def __init__(self, configs: ConfigsResource) -> None:
        self._configs = configs

        self.create = to_raw_response_wrapper(
            configs.create,
        )
        self.retrieve = to_raw_response_wrapper(
            configs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            configs.list,
        )
        self.delete = to_raw_response_wrapper(
            configs.delete,
        )
        self.execute = to_raw_response_wrapper(
            configs.execute,
        )
        self.retrieve_details = to_raw_response_wrapper(
            configs.retrieve_details,
        )


class AsyncConfigsResourceWithRawResponse:
    def __init__(self, configs: AsyncConfigsResource) -> None:
        self._configs = configs

        self.create = async_to_raw_response_wrapper(
            configs.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            configs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            configs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            configs.delete,
        )
        self.execute = async_to_raw_response_wrapper(
            configs.execute,
        )
        self.retrieve_details = async_to_raw_response_wrapper(
            configs.retrieve_details,
        )


class ConfigsResourceWithStreamingResponse:
    def __init__(self, configs: ConfigsResource) -> None:
        self._configs = configs

        self.create = to_streamed_response_wrapper(
            configs.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            configs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            configs.list,
        )
        self.delete = to_streamed_response_wrapper(
            configs.delete,
        )
        self.execute = to_streamed_response_wrapper(
            configs.execute,
        )
        self.retrieve_details = to_streamed_response_wrapper(
            configs.retrieve_details,
        )


class AsyncConfigsResourceWithStreamingResponse:
    def __init__(self, configs: AsyncConfigsResource) -> None:
        self._configs = configs

        self.create = async_to_streamed_response_wrapper(
            configs.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            configs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            configs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            configs.delete,
        )
        self.execute = async_to_streamed_response_wrapper(
            configs.execute,
        )
        self.retrieve_details = async_to_streamed_response_wrapper(
            configs.retrieve_details,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._compat import PYDANTIC_V2
from .._models import BaseModel
from .unary_condition import UnaryCondition
from .node_item_output import NodeItemOutput
from .plan_config_output import PlanConfigOutput
from .workflow_config_output import WorkflowConfigOutput
from .compound_condition_output import CompoundConditionOutput

__all__ = [
    "ConfigListResponse",
    "ConfigListResponseItem",
    "ConfigListResponseItemStateMachineConfigOutput",
    "ConfigListResponseItemStateMachineConfigOutputMachine",
    "ConfigListResponseItemStateMachineConfigOutputMachineNextNode",
    "ConfigListResponseItemStateMachineConfigOutputMachineNextNodeCase",
    "ConfigListResponseItemStateMachineConfigOutputMachineNextNodeCaseCondition",
]

ConfigListResponseItemStateMachineConfigOutputMachineNextNodeCaseCondition: TypeAlias = Union[
    UnaryCondition, "CompoundConditionOutput"
]


class ConfigListResponseItemStateMachineConfigOutputMachineNextNodeCase(BaseModel):
    condition: ConfigListResponseItemStateMachineConfigOutputMachineNextNodeCaseCondition
    """Representation of a boolean function with a single input e.g.

    the condition specified by input_name: x operator: 'contains' ref_value: 'c'
    would evaluate to True if x == 'cat' Operators are defined in the constant
    function store CONDITIONAL_ACTION_MAP
    """

    value: str


class ConfigListResponseItemStateMachineConfigOutputMachineNextNode(BaseModel):
    default: str

    cases: Optional[List[ConfigListResponseItemStateMachineConfigOutputMachineNextNodeCase]] = None


class ConfigListResponseItemStateMachineConfigOutputMachine(BaseModel):
    next_node: ConfigListResponseItemStateMachineConfigOutputMachineNextNode
    """A switch statement for state machines to select the next state to execute."""

    workflow_config: "WorkflowConfigOutput"

    write_to_state: Optional[Dict[str, str]] = None


class ConfigListResponseItemStateMachineConfigOutput(BaseModel):
    machine: Dict[str, ConfigListResponseItemStateMachineConfigOutputMachine]

    starting_node: str

    id: Optional[str] = None

    account_id: Optional[str] = None

    application_variant_id: Optional[str] = None

    base_url: Optional[str] = None

    concurrency_default: Optional[bool] = None

    datasets: Optional[List[object]] = None

    done_string: Optional[str] = None

    egp_api_key_override: Optional[str] = None

    egp_ui_evaluation: Optional[object] = None

    evaluations: Optional[List["NodeItemOutput"]] = None

    final_output_nodes: Optional[List[str]] = None

    initial_state: Optional[object] = None

    nodes_to_log: Union[str, List[str], None] = None

    num_workers: Optional[int] = None

    streaming_nodes: Optional[List[str]] = None

    type: Optional[Literal["workflow", "plan", "state_machine"]] = None


ConfigListResponseItem: TypeAlias = Union[
    PlanConfigOutput, WorkflowConfigOutput, ConfigListResponseItemStateMachineConfigOutput
]

ConfigListResponse: TypeAlias = List[ConfigListResponseItem]

if PYDANTIC_V2:
    ConfigListResponseItemStateMachineConfigOutput.model_rebuild()
    ConfigListResponseItemStateMachineConfigOutputMachine.model_rebuild()
    ConfigListResponseItemStateMachineConfigOutputMachineNextNode.model_rebuild()
    ConfigListResponseItemStateMachineConfigOutputMachineNextNodeCase.model_rebuild()
else:
    ConfigListResponseItemStateMachineConfigOutput.update_forward_refs()  # type: ignore
    ConfigListResponseItemStateMachineConfigOutputMachine.update_forward_refs()  # type: ignore
    ConfigListResponseItemStateMachineConfigOutputMachineNextNode.update_forward_refs()  # type: ignore
    ConfigListResponseItemStateMachineConfigOutputMachineNextNodeCase.update_forward_refs()  # type: ignore



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .unary_condition_param import UnaryConditionParam

__all__ = ["CompoundConditionInputParam", "Condition"]

Condition: TypeAlias = Union[UnaryConditionParam, "CompoundConditionInputParam"]


class CompoundConditionInputParam(TypedDict, total=False):
    conditions: Required[Optional[Iterable[Condition]]]

    input_names: List[str]

    logical_operator: Literal["ALL", "ANY", "NOT"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ConfigExecuteParams", "Message"]


class ConfigExecuteParams(TypedDict, total=False):
    messages: Required[Optional[Iterable[Message]]]

    metadata: Required[Optional[object]]

    run_id: Required[Optional[str]]

    session_id: Required[str]

    id: str

    concurrent: Optional[bool]

    stream: bool


class Message(TypedDict, total=False):
    content: Required[str]

    role: Required[Literal["assistant", "user", "system"]]

    uuid: Annotated[str, PropertyInfo(alias="UUID")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .node_item_param import NodeItemParam

__all__ = [
    "PlanConfigParam",
    "Plan",
    "PlanWorkflowItem",
    "PlanBranchConfig",
    "PlanBranchConfigConditionalWorkflow",
    "PlanBranchConfigConditionalWorkflowConditionTree",
    "PlanBranchConfigConditionalWorkflowConditionTreeCondition",
    "PlanBranchConfigConditionalWorkflowConditionTreeConditionUnaryCondition",
    "PlanBranchConfigConditionalWorkflowConditionTreeConditionCompoundCondition",
    "PlanBranchConfigConditionalWorkflowConditionTreeConditionCompoundConditionCondition",
    "PlanBranchConfigConditionalWorkflowConditionTreeConditionCompoundConditionConditionUnaryCondition",
    "PlanLoopConfig",
    "PlanLoopConfigCondition",
    "PlanLoopConfigConditionCondition",
    "PlanLoopConfigConditionConditionUnaryCondition",
    "PlanLoopConfigConditionConditionCompoundCondition",
    "PlanLoopConfigConditionConditionCompoundConditionCondition",
    "PlanLoopConfigConditionConditionCompoundConditionConditionUnaryCondition",
    "PlanLoopConfigWorkflow",
    "PlanLoopConfigLoopInputs",
    "PlanLoopConfigLoopInputsSelfEdge",
    "Workflows",
    "WorkflowsWorkflow",
]


class PlanWorkflowItem(TypedDict, total=False):
    workflow_name: Required[str]

    request_user_input: Literal["never", "maybe", "always"]
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """

    workflow_alias: str

    workflow_inputs: Dict[str, Union[str, Dict[str, Union[str, object]]]]

    workflow_save_outputs: Dict[str, str]


class PlanBranchConfigConditionalWorkflowConditionTreeConditionUnaryCondition(TypedDict, total=False):
    condition_input_var: Required[str]

    operator: Required[str]

    reference_var: object


class PlanBranchConfigConditionalWorkflowConditionTreeConditionCompoundConditionConditionUnaryCondition(
    TypedDict, total=False
):
    condition_input_var: Required[str]

    operator: Required[str]

    reference_var: object


PlanBranchConfigConditionalWorkflowConditionTreeConditionCompoundConditionCondition: TypeAlias = Union[
    PlanBranchConfigConditionalWorkflowConditionTreeConditionCompoundConditionConditionUnaryCondition, object
]


class PlanBranchConfigConditionalWorkflowConditionTreeConditionCompoundCondition(TypedDict, total=False):
    conditions: Iterable[PlanBranchConfigConditionalWorkflowConditionTreeConditionCompoundConditionCondition]

    input_names: List[str]

    logical_operator: Literal["ALL", "ANY", "NOT"]


PlanBranchConfigConditionalWorkflowConditionTreeCondition: TypeAlias = Union[
    PlanBranchConfigConditionalWorkflowConditionTreeConditionUnaryCondition,
    PlanBranchConfigConditionalWorkflowConditionTreeConditionCompoundCondition,
]


class PlanBranchConfigConditionalWorkflowConditionTree(TypedDict, total=False):
    conditions: Iterable[PlanBranchConfigConditionalWorkflowConditionTreeCondition]

    input_names: List[str]

    logical_operator: Literal["ALL", "ANY", "NOT"]


class PlanBranchConfigConditionalWorkflow(TypedDict, total=False):
    condition: Required[Literal["if", "elif", "else"]]

    workflow_name: Required[str]

    condition_input_var: str

    condition_tree: PlanBranchConfigConditionalWorkflowConditionTree
    """Representation of a compound boolean statement, i.e.

    a negation, conjunction, or disjunction of UnaryConditions
    """

    operator: str

    reference_var: str

    request_user_input: Literal["never", "maybe", "always"]
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """

    workflow_alias: str

    workflow_inputs: Dict[str, Union[str, Dict[str, Union[str, object]]]]

    workflow_nodes: List[str]

    workflow_save_outputs: Dict[str, str]


class PlanBranchConfig(TypedDict, total=False):
    branch: Required[str]

    conditional_workflows: Required[Iterable[PlanBranchConfigConditionalWorkflow]]

    merge_outputs: Dict[str, List[str]]

    request_user_input: Literal["never", "maybe", "always"]
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """

    save_merge_to_memory: Dict[str, str]


class PlanLoopConfigConditionConditionUnaryCondition(TypedDict, total=False):
    condition_input_var: Required[str]

    operator: Required[str]

    reference_var: object


class PlanLoopConfigConditionConditionCompoundConditionConditionUnaryCondition(TypedDict, total=False):
    condition_input_var: Required[str]

    operator: Required[str]

    reference_var: object


PlanLoopConfigConditionConditionCompoundConditionCondition: TypeAlias = Union[
    PlanLoopConfigConditionConditionCompoundConditionConditionUnaryCondition, object
]


class PlanLoopConfigConditionConditionCompoundCondition(TypedDict, total=False):
    conditions: Iterable[PlanLoopConfigConditionConditionCompoundConditionCondition]

    input_names: List[str]

    logical_operator: Literal["ALL", "ANY", "NOT"]


PlanLoopConfigConditionCondition: TypeAlias = Union[
    PlanLoopConfigConditionConditionUnaryCondition, PlanLoopConfigConditionConditionCompoundCondition
]


class PlanLoopConfigCondition(TypedDict, total=False):
    conditions: Iterable[PlanLoopConfigConditionCondition]

    input_names: List[str]

    logical_operator: Literal["ALL", "ANY", "NOT"]


class PlanLoopConfigWorkflow(TypedDict, total=False):
    workflow_name: Required[str]

    request_user_input: Literal["never", "maybe", "always"]
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """

    workflow_alias: str

    workflow_inputs: Dict[str, Union[str, Dict[str, Union[str, object]]]]

    workflow_save_outputs: Dict[str, str]


class PlanLoopConfigLoopInputsSelfEdge(TypedDict, total=False):
    node_name: Required[str]

    default_source: str

    default_value: object


PlanLoopConfigLoopInputs: TypeAlias = Union[str, PlanLoopConfigLoopInputsSelfEdge]


class PlanLoopConfig(TypedDict, total=False):
    condition: Required[PlanLoopConfigCondition]
    """Representation of a compound boolean statement, i.e.

    a negation, conjunction, or disjunction of UnaryConditions
    """

    max_iter: Required[int]

    name: Required[str]

    workflow: Required[PlanLoopConfigWorkflow]
    """
    Representation of an instance of an abstract Workflow Attributes: workflow_name:
    Key in the map of workflows defined at the top of a plan config workflow_alias:
    Alias of the abstract workflow in the graph. If None, defaults to workflow_name.
    Use in order to re-use the same abstract workflow in multiple portions of the
    graph. workflow_inputs: Inputs to the workflow can be: 1) empty if this workflow
    does not receive input from another workflow, 2) a dictionary mapping another
    workflow's outputs or a branch's merged outputs to this workflows input keys For
    case 2, inputs can be a mapping from: a) str to str b) str to dict(str, str) c)
    str to dict(str, dict(str, str)) workflow_save_outputs: If the following
    parameter is set, the node output can ADDITIONALLY be stored under an alias in
    memory. request_user_input: request_user_input behaves as follows: - When set to
    NEVER, input of this step must be available in memory otherwise exception will
    be raised - When set to NEED_BASED, run will _only_ be interrupted when reaching
    this step in order to request required input fields in case they don't already
    exist in memory from previous node outputs or user inputs - When set to ALWAYS,
    run will _always_ be interrupted when reaching this step in order to request
    these input fields, whether they exist in memory or not
    """

    loop_inputs: Dict[str, PlanLoopConfigLoopInputs]

    merge_outputs: Dict[str, str]

    request_user_input: Literal["never", "maybe", "always"]
    """
    If always: available the first time: request missing the first time: request
    available the next time: continue missing the next time: request (treat like
    need_based) If need_based: available the first time: continue missing the first
    time: request available the next time: continue missing the next time: request
    (treat like the first time) If never: available the first time: continue missing
    the first time: raise Exception available the next time: continue missing the
    next time: raise Exception
    """


Plan: TypeAlias = Union[PlanWorkflowItem, PlanBranchConfig, PlanLoopConfig]


class WorkflowsWorkflow(TypedDict, total=False):
    name: Required[str]

    nodes: Required[Iterable[NodeItemParam]]


Workflows: TypeAlias = Union[WorkflowsWorkflow, str, Iterable[NodeItemParam]]


class PlanConfigParam(TypedDict, total=False):
    plan: Required[Iterable[Plan]]

    id: str

    account_id: str

    application_variant_id: str

    base_url: str

    concurrency_default: bool

    datasets: Iterable[object]

    egp_api_key_override: str

    egp_ui_evaluation: object

    evaluations: Iterable[NodeItemParam]

    final_output_nodes: List[str]

    nodes_to_log: Union[str, List[str]]

    num_workers: int

    streaming_nodes: List[str]

    type: Literal["workflow", "plan", "state_machine"]
    """An enumeration."""

    workflows: Dict[str, Workflows]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Literal, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = ["ParameterTypeInputParam", "AdditionalProperties", "AnyOf", "Items"]

AdditionalProperties: TypeAlias = Union[
    Literal["object", "array"], Literal["string", "number", "integer", "boolean", "null"], "ParameterTypeInputParam"
]

AnyOf: TypeAlias = Union[
    Literal["object", "array"], Literal["string", "number", "integer", "boolean", "null"], "ParameterTypeInputParam"
]

Items: TypeAlias = Union[
    Literal["object", "array"], Literal["string", "number", "integer", "boolean", "null"], "ParameterTypeInputParam"
]


class ParameterTypeInputParam(TypedDict, total=False):
    additional_properties: Annotated[Optional[AdditionalProperties], PropertyInfo(alias="additionalProperties")]

    any_of: Annotated[Optional[Iterable[AnyOf]], PropertyInfo(alias="anyOf")]

    items: Optional[Items]

    property_names: Annotated[
        Union[
            Literal["string", "number", "integer", "boolean", "null"],
            List[Literal["string", "number", "integer", "boolean", "null"]],
            None,
        ],
        PropertyInfo(alias="propertyNames"),
    ]

    type: Union[
        Literal["object", "array"],
        Literal["string", "number", "integer", "boolean", "null"],
        List[Literal["object", "array", "string", "number", "integer", "boolean", "null"]],
        None,
    ]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "NodeItemParam",
    "Config",
    "ConfigNodeConfig",
    "ConfigJinjaNodeConfig",
    "ConfigJinjaNodeConfigDataTransformations",
    "ConfigJinjaNodeConfigOutputTemplate",
    "ConfigChunkEvaluationNodeConfig",
    "ConfigRerankerNodeConfig",
    "ConfigRetrieverNodeConfig",
    "ConfigCitationNodeConfig",
    "ConfigCitationNodeConfigCitationContext",
    "ConfigSearchCitationNodeConfig",
    "ConfigDataTransformNodeConfig",
    "ConfigCreateMessagesNodeConfig",
    "ConfigCreateMessagesNodeConfigMessageConfig",
    "ConfigCreateMessagesNodeConfigMessageConfigAlternatingRoleMessages",
    "ConfigCreateMessagesNodeConfigMessageConfigSingleRoleMessages",
    "ConfigInsertMessagesConfig",
    "ConfigRemoveMessageConfig",
    "ConfigTokenizerChatTemplateConfig",
    "ConfigLlmEngineNodeConfig",
    "ConfigLlmEngineNodeConfigBatchSysKwargs",
    "ConfigResponseParserNodeConfig",
    "ConfigProcessingNodeConfig",
    "ConfigProcessingNodeConfigFunctionSpecs",
    "ConfigSqlExecutorNodeConfig",
    "ConfigStaticNodeConfig",
    "ConfigGenerationNodeConfig",
]


class ConfigNodeConfig(TypedDict, total=False):
    node_metadata: List[str]

    num_workers: int


class ConfigJinjaNodeConfigDataTransformations(TypedDict, total=False):
    jinja_helper_functions: List[Union[str, object]]

    jinja_template_path: str

    jinja_template_str: str
    """Raw template to apply to the data.

    This should be a Jinja2 template string. Please note, the data will be mapped as
    'value' in the template. Default None corresponds to {{value}}. Should access
    property `jinja_template_str` or field `jinja_template_str_loaded` for the
    loaded template data
    """

    jinja_template_str_loaded: str
    """
    The original jinja_template_str field from the config might not contain the
    needed template, and we may need to load S3 data specified with
    `jinja_template_path`. This field caches the loaded template content, it is also
    accessed through property `jinja_template_str`.
    """


class ConfigJinjaNodeConfigOutputTemplate(TypedDict, total=False):
    jinja_helper_functions: List[Union[str, object]]

    jinja_template_path: str

    jinja_template_str: str
    """Raw template to apply to the data.

    This should be a Jinja2 template string. Please note, the data will be mapped as
    'value' in the template. Default None corresponds to {{value}}. Should access
    property `jinja_template_str` or field `jinja_template_str_loaded` for the
    loaded template data
    """

    jinja_template_str_loaded: str
    """
    The original jinja_template_str field from the config might not contain the
    needed template, and we may need to load S3 data specified with
    `jinja_template_path`. This field caches the loaded template content, it is also
    accessed through property `jinja_template_str`.
    """


class ConfigJinjaNodeConfig(TypedDict, total=False):
    context_chunks_key: str

    data_transformations: Dict[str, ConfigJinjaNodeConfigDataTransformations]

    llm_model: str

    log_output: bool

    log_prefix: str

    max_tokens: int

    node_metadata: List[str]

    num_workers: int

    output_template: ConfigJinjaNodeConfigOutputTemplate
    """
    Base model for a Jinja template. Guaranteed to store a string that can be read
    in to Template().
    """

    verbose: bool


class ConfigChunkEvaluationNodeConfig(TypedDict, total=False):
    top_k_thresholds: Required[Iterable[int]]

    fuzzy_match_threshold: float

    node_metadata: List[str]

    num_workers: int

    require_all: bool


class ConfigRerankerNodeConfig(TypedDict, total=False):
    num_to_return: Required[int]

    scorers: Required[Iterable[object]]

    node_metadata: List[str]

    num_workers: int

    score_threshold: float


class ConfigRetrieverNodeConfig(TypedDict, total=False):
    num_to_return: Required[int]

    exact_knn_search: bool

    knowledge_base_id: str

    knowledge_base_name: str

    metadata: Dict[str, str]

    node_metadata: List[str]

    num_workers: int


class ConfigCitationNodeConfigCitationContext(TypedDict, total=False):
    generate_with_llm: bool

    metric: str

    min_similarity: float

    score: Literal["precision", "recall", "fmeasure"]


class ConfigCitationNodeConfig(TypedDict, total=False):
    citation_type: Required[Literal["rouge", "model_defined"]]

    citation_context: ConfigCitationNodeConfigCitationContext

    node_metadata: List[str]

    num_workers: int

    s3_path_override: str


class ConfigSearchCitationNodeConfig(TypedDict, total=False):
    end_search_regex: Required[str]

    search_regex: Required[str]

    node_metadata: List[str]

    num_workers: int


class ConfigDataTransformNodeConfig(TypedDict, total=False):
    action: Required[str]

    additional_inputs: object

    apply_to_dictlist_leaves: bool

    node_metadata: List[str]

    num_workers: int


class ConfigCreateMessagesNodeConfigMessageConfigAlternatingRoleMessages(TypedDict, total=False):
    role_value_pairs: Required[Iterable[Dict[str, str]]]


class ConfigCreateMessagesNodeConfigMessageConfigSingleRoleMessages(TypedDict, total=False):
    content: Required[str]

    role: Required[str]


ConfigCreateMessagesNodeConfigMessageConfig: TypeAlias = Union[
    ConfigCreateMessagesNodeConfigMessageConfigAlternatingRoleMessages,
    ConfigCreateMessagesNodeConfigMessageConfigSingleRoleMessages,
]


class ConfigCreateMessagesNodeConfig(TypedDict, total=False):
    message_configs: Required[Iterable[ConfigCreateMessagesNodeConfigMessageConfig]]

    node_metadata: List[str]

    num_workers: int


class ConfigInsertMessagesConfig(TypedDict, total=False):
    index: Required[int]

    node_metadata: List[str]

    num_workers: int


class ConfigRemoveMessageConfig(TypedDict, total=False):
    index: Required[int]

    node_metadata: List[str]

    num_workers: int


class ConfigTokenizerChatTemplateConfig(TypedDict, total=False):
    llm_model: Required[str]

    add_generation_prompt: bool

    kwargs: object

    max_length: int

    node_metadata: List[str]

    num_workers: int

    padding: bool

    truncation: bool


class ConfigLlmEngineNodeConfigBatchSysKwargs(TypedDict, total=False):
    checkpoint_path: str

    labels: Dict[str, str]

    num_shards: int

    seed: int


class ConfigLlmEngineNodeConfig(TypedDict, total=False):
    llm_model: Required[str]

    batch_run_mode: Literal["sync", "async"]

    batch_sys_kwargs: ConfigLlmEngineNodeConfigBatchSysKwargs

    frequency_penalty: float

    guided_choice: List[str]

    guided_json: object

    guided_regex: str

    include_stop_str_in_output: bool

    max_tokens: int

    node_metadata: List[str]

    num_workers: int

    presence_penalty: float

    stop_sequences: List[str]

    temperature: float

    timeout: int

    top_k: int

    top_p: float


class ConfigResponseParserNodeConfig(TypedDict, total=False):
    action: Required[str]

    node_metadata: List[str]

    num_workers: int

    reference_value: object


class ConfigProcessingNodeConfigFunctionSpecs(TypedDict, total=False):
    kwargs: Required[object]

    path: Required[str]


class ConfigProcessingNodeConfig(TypedDict, total=False):
    function_specs: Required[Dict[str, ConfigProcessingNodeConfigFunctionSpecs]]

    return_key: Required[str]

    node_metadata: List[str]

    num_workers: int


class ConfigSqlExecutorNodeConfig(TypedDict, total=False):
    connector_kwargs: Required[Dict[str, str]]

    connector_type: Literal["snowflake"]

    log_queries: bool

    node_metadata: List[str]

    num_workers: int

    return_type: Literal["df", "dicts", "markdown", "json", "str"]

    schema_remapping_file: str

    secrets: List[str]


class ConfigStaticNodeConfig(TypedDict, total=False):
    from_file: Union[Iterable[object], str, object]

    node_metadata: List[str]

    num_workers: int

    value: object


class ConfigGenerationNodeConfig(TypedDict, total=False):
    llm_model: str

    llm_model_deployment: str

    llm_model_instance: str

    max_tokens: int

    node_metadata: List[str]

    num_workers: int

    stop_sequences: List[str]

    strip_whitespace: bool

    temperature: float

    tool_name: str


Config: TypeAlias = Union[
    ConfigNodeConfig,
    ConfigJinjaNodeConfig,
    ConfigChunkEvaluationNodeConfig,
    ConfigRerankerNodeConfig,
    ConfigRetrieverNodeConfig,
    ConfigCitationNodeConfig,
    ConfigSearchCitationNodeConfig,
    ConfigDataTransformNodeConfig,
    ConfigCreateMessagesNodeConfig,
    ConfigInsertMessagesConfig,
    ConfigRemoveMessageConfig,
    ConfigTokenizerChatTemplateConfig,
    ConfigLlmEngineNodeConfig,
    ConfigResponseParserNodeConfig,
    ConfigProcessingNodeConfig,
    ConfigSqlExecutorNodeConfig,
    ConfigStaticNodeConfig,
    ConfigGenerationNodeConfig,
]


class NodeItemParam(TypedDict, total=False):
    config: Required[Config]
    """A data model describing parameters for back-citation using ROUGE similarity.

    metric is the ROUGE metric to use (e.g. rouge1, rouge2, rougeLsum) score is one
    of "precision", "recall", "fmeasure"

    NOTE (john): copied directly from generation.py in order to subclass from
    NodeConfig.
    """

    name: Required[str]

    type: Required[str]

    inputs: Dict[str, Union[str, Dict[str, Union[str, object]]]]

    save_to_memory_as: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from sgp_agents import Agents, AsyncAgents
from tests.utils import assert_matches_type
from sgp_agents.types import (
    ConfigListResponse,
    ConfigDeleteResponse,
    ConfigRetrieveResponse,
    ConfigRetrieveDetailsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestConfigs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Agents) -> None:
        config = client.configs.create(
            config={
                "plan": [
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                ]
            },
        )
        assert_matches_type(object, config, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Agents) -> None:
        config = client.configs.create(
            config={
                "plan": [
                    {
                        "workflow_name": "workflow_name",
                        "request_user_input": "never",
                        "workflow_alias": "workflow_alias",
                        "workflow_inputs": {"foo": "string"},
                        "workflow_save_outputs": {"foo": "string"},
                    },
                    {
                        "workflow_name": "workflow_name",
                        "request_user_input": "never",
                        "workflow_alias": "workflow_alias",
                        "workflow_inputs": {"foo": "string"},
                        "workflow_save_outputs": {"foo": "string"},
                    },
                    {
                        "workflow_name": "workflow_name",
                        "request_user_input": "never",
                        "workflow_alias": "workflow_alias",
                        "workflow_inputs": {"foo": "string"},
                        "workflow_save_outputs": {"foo": "string"},
                    },
                ],
                "id": "id",
                "account_id": "account_id",
                "application_variant_id": "application_variant_id",
                "base_url": "base_url",
                "concurrency_default": True,
                "datasets": [{}, {}, {}],
                "egp_api_key_override": "egp_api_key_override",
                "egp_ui_evaluation": {},
                "evaluations": [
                    {
                        "config": {
                            "node_metadata": ["string", "string", "string"],
                            "num_workers": 0,
                            "type_hints": {
                                "foo": {
                                    "additional_properties": "object",
                                    "any_of": ["object", "object", "object"],
                                    "items": "object",
                                    "property_names": "string",
                                    "type": "object",
                                }
                            },
                        },
                        "name": "name",
                        "type": "type",
                        "inputs": {"foo": "string"},
                        "save_to_memory_as": "save_to_memory_as",
                    },
                    {
                        "config": {
                            "node_metadata": ["string", "string", "string"],
                            "num_workers": 0,
                            "type_hints": {
                                "foo": {
                                    "additional_properties": "object",
                                    "any_of": ["object", "object", "object"],
                                    "items": "object",
                                    "property_names": "string",
                                    "type": "object",
                                }
                            },
                        },
                        "name": "name",
                        "type": "type",
                        "inputs": {"foo": "string"},
                        "save_to_memory_as": "save_to_memory_as",
                    },
                    {
                        "config": {
                            "node_metadata": ["string", "string", "string"],
                            "num_workers": 0,
                            "type_hints": {
                                "foo": {
                                    "additional_properties": "object",
                                    "any_of": ["object", "object", "object"],
                                    "items": "object",
                                    "property_names": "string",
                                    "type": "object",
                                }
                            },
                        },
                        "name": "name",
                        "type": "type",
                        "inputs": {"foo": "string"},
                        "save_to_memory_as": "save_to_memory_as",
                    },
                ],
                "final_output_nodes": ["string", "string", "string"],
                "nodes_to_log": "string",
                "num_workers": 0,
                "streaming_nodes": ["string", "string", "string"],
                "type": "workflow",
                "workflows": {
                    "foo": {
                        "name": "name",
                        "nodes": [
                            {
                                "config": {
                                    "node_metadata": ["string", "string", "string"],
                                    "num_workers": 0,
                                    "type_hints": {
                                        "foo": {
                                            "additional_properties": "object",
                                            "any_of": ["object", "object", "object"],
                                            "items": "object",
                                            "property_names": "string",
                                            "type": "object",
                                        }
                                    },
                                },
                                "name": "name",
                                "type": "type",
                                "inputs": {"foo": "string"},
                                "save_to_memory_as": "save_to_memory_as",
                            },
                            {
                                "config": {
                                    "node_metadata": ["string", "string", "string"],
                                    "num_workers": 0,
                                    "type_hints": {
                                        "foo": {
                                            "additional_properties": "object",
                                            "any_of": ["object", "object", "object"],
                                            "items": "object",
                                            "property_names": "string",
                                            "type": "object",
                                        }
                                    },
                                },
                                "name": "name",
                                "type": "type",
                                "inputs": {"foo": "string"},
                                "save_to_memory_as": "save_to_memory_as",
                            },
                            {
                                "config": {
                                    "node_metadata": ["string", "string", "string"],
                                    "num_workers": 0,
                                    "type_hints": {
                                        "foo": {
                                            "additional_properties": "object",
                                            "any_of": ["object", "object", "object"],
                                            "items": "object",
                                            "property_names": "string",
                                            "type": "object",
                                        }
                                    },
                                },
                                "name": "name",
                                "type": "type",
                                "inputs": {"foo": "string"},
                                "save_to_memory_as": "save_to_memory_as",
                            },
                        ],
                    }
                },
            },
        )
        assert_matches_type(object, config, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Agents) -> None:
        response = client.configs.with_raw_response.create(
            config={
                "plan": [
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                ]
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = response.parse()
        assert_matches_type(object, config, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Agents) -> None:
        with client.configs.with_streaming_response.create(
            config={
                "plan": [
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                ]
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = response.parse()
            assert_matches_type(object, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Agents) -> None:
        config = client.configs.retrieve(
            "config_id",
        )
        assert_matches_type(ConfigRetrieveResponse, config, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Agents) -> None:
        response = client.configs.with_raw_response.retrieve(
            "config_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = response.parse()
        assert_matches_type(ConfigRetrieveResponse, config, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Agents) -> None:
        with client.configs.with_streaming_response.retrieve(
            "config_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = response.parse()
            assert_matches_type(ConfigRetrieveResponse, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Agents) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_id` but received ''"):
            client.configs.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: Agents) -> None:
        config = client.configs.list(
            config_type="workflow",
        )
        assert_matches_type(ConfigListResponse, config, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Agents) -> None:
        config = client.configs.list(
            config_type="workflow",
            account_id="account_id",
        )
        assert_matches_type(ConfigListResponse, config, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Agents) -> None:
        response = client.configs.with_raw_response.list(
            config_type="workflow",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = response.parse()
        assert_matches_type(ConfigListResponse, config, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Agents) -> None:
        with client.configs.with_streaming_response.list(
            config_type="workflow",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = response.parse()
            assert_matches_type(ConfigListResponse, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Agents) -> None:
        config = client.configs.delete(
            "config_id",
        )
        assert_matches_type(ConfigDeleteResponse, config, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Agents) -> None:
        response = client.configs.with_raw_response.delete(
            "config_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = response.parse()
        assert_matches_type(ConfigDeleteResponse, config, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Agents) -> None:
        with client.configs.with_streaming_response.delete(
            "config_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = response.parse()
            assert_matches_type(ConfigDeleteResponse, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Agents) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_id` but received ''"):
            client.configs.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_execute(self, client: Agents) -> None:
        config = client.configs.execute(
            config_id="config_id",
            messages=[
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
            ],
            metadata={},
            run_id="run_id",
            session_id="session_id",
        )
        assert_matches_type(object, config, path=["response"])

    @parametrize
    def test_method_execute_with_all_params(self, client: Agents) -> None:
        config = client.configs.execute(
            config_id="config_id",
            messages=[
                {
                    "content": "content",
                    "role": "assistant",
                    "uuid": "UUID",
                },
                {
                    "content": "content",
                    "role": "assistant",
                    "uuid": "UUID",
                },
                {
                    "content": "content",
                    "role": "assistant",
                    "uuid": "UUID",
                },
            ],
            metadata={},
            run_id="run_id",
            session_id="session_id",
            id="id",
            concurrent=True,
            stream=True,
        )
        assert_matches_type(object, config, path=["response"])

    @parametrize
    def test_raw_response_execute(self, client: Agents) -> None:
        response = client.configs.with_raw_response.execute(
            config_id="config_id",
            messages=[
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
            ],
            metadata={},
            run_id="run_id",
            session_id="session_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = response.parse()
        assert_matches_type(object, config, path=["response"])

    @parametrize
    def test_streaming_response_execute(self, client: Agents) -> None:
        with client.configs.with_streaming_response.execute(
            config_id="config_id",
            messages=[
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
            ],
            metadata={},
            run_id="run_id",
            session_id="session_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = response.parse()
            assert_matches_type(object, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_execute(self, client: Agents) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_id` but received ''"):
            client.configs.with_raw_response.execute(
                config_id="",
                messages=[
                    {
                        "content": "content",
                        "role": "assistant",
                    },
                    {
                        "content": "content",
                        "role": "assistant",
                    },
                    {
                        "content": "content",
                        "role": "assistant",
                    },
                ],
                metadata={},
                run_id="run_id",
                session_id="session_id",
            )

    @parametrize
    def test_method_retrieve_details(self, client: Agents) -> None:
        config = client.configs.retrieve_details(
            "config_id",
        )
        assert_matches_type(ConfigRetrieveDetailsResponse, config, path=["response"])

    @parametrize
    def test_raw_response_retrieve_details(self, client: Agents) -> None:
        response = client.configs.with_raw_response.retrieve_details(
            "config_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = response.parse()
        assert_matches_type(ConfigRetrieveDetailsResponse, config, path=["response"])

    @parametrize
    def test_streaming_response_retrieve_details(self, client: Agents) -> None:
        with client.configs.with_streaming_response.retrieve_details(
            "config_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = response.parse()
            assert_matches_type(ConfigRetrieveDetailsResponse, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve_details(self, client: Agents) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_id` but received ''"):
            client.configs.with_raw_response.retrieve_details(
                "",
            )


class TestAsyncConfigs:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncAgents) -> None:
        config = await async_client.configs.create(
            config={
                "plan": [
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                ]
            },
        )
        assert_matches_type(object, config, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncAgents) -> None:
        config = await async_client.configs.create(
            config={
                "plan": [
                    {
                        "workflow_name": "workflow_name",
                        "request_user_input": "never",
                        "workflow_alias": "workflow_alias",
                        "workflow_inputs": {"foo": "string"},
                        "workflow_save_outputs": {"foo": "string"},
                    },
                    {
                        "workflow_name": "workflow_name",
                        "request_user_input": "never",
                        "workflow_alias": "workflow_alias",
                        "workflow_inputs": {"foo": "string"},
                        "workflow_save_outputs": {"foo": "string"},
                    },
                    {
                        "workflow_name": "workflow_name",
                        "request_user_input": "never",
                        "workflow_alias": "workflow_alias",
                        "workflow_inputs": {"foo": "string"},
                        "workflow_save_outputs": {"foo": "string"},
                    },
                ],
                "id": "id",
                "account_id": "account_id",
                "application_variant_id": "application_variant_id",
                "base_url": "base_url",
                "concurrency_default": True,
                "datasets": [{}, {}, {}],
                "egp_api_key_override": "egp_api_key_override",
                "egp_ui_evaluation": {},
                "evaluations": [
                    {
                        "config": {
                            "node_metadata": ["string", "string", "string"],
                            "num_workers": 0,
                            "type_hints": {
                                "foo": {
                                    "additional_properties": "object",
                                    "any_of": ["object", "object", "object"],
                                    "items": "object",
                                    "property_names": "string",
                                    "type": "object",
                                }
                            },
                        },
                        "name": "name",
                        "type": "type",
                        "inputs": {"foo": "string"},
                        "save_to_memory_as": "save_to_memory_as",
                    },
                    {
                        "config": {
                            "node_metadata": ["string", "string", "string"],
                            "num_workers": 0,
                            "type_hints": {
                                "foo": {
                                    "additional_properties": "object",
                                    "any_of": ["object", "object", "object"],
                                    "items": "object",
                                    "property_names": "string",
                                    "type": "object",
                                }
                            },
                        },
                        "name": "name",
                        "type": "type",
                        "inputs": {"foo": "string"},
                        "save_to_memory_as": "save_to_memory_as",
                    },
                    {
                        "config": {
                            "node_metadata": ["string", "string", "string"],
                            "num_workers": 0,
                            "type_hints": {
                                "foo": {
                                    "additional_properties": "object",
                                    "any_of": ["object", "object", "object"],
                                    "items": "object",
                                    "property_names": "string",
                                    "type": "object",
                                }
                            },
                        },
                        "name": "name",
                        "type": "type",
                        "inputs": {"foo": "string"},
                        "save_to_memory_as": "save_to_memory_as",
                    },
                ],
                "final_output_nodes": ["string", "string", "string"],
                "nodes_to_log": "string",
                "num_workers": 0,
                "streaming_nodes": ["string", "string", "string"],
                "type": "workflow",
                "workflows": {
                    "foo": {
                        "name": "name",
                        "nodes": [
                            {
                                "config": {
                                    "node_metadata": ["string", "string", "string"],
                                    "num_workers": 0,
                                    "type_hints": {
                                        "foo": {
                                            "additional_properties": "object",
                                            "any_of": ["object", "object", "object"],
                                            "items": "object",
                                            "property_names": "string",
                                            "type": "object",
                                        }
                                    },
                                },
                                "name": "name",
                                "type": "type",
                                "inputs": {"foo": "string"},
                                "save_to_memory_as": "save_to_memory_as",
                            },
                            {
                                "config": {
                                    "node_metadata": ["string", "string", "string"],
                                    "num_workers": 0,
                                    "type_hints": {
                                        "foo": {
                                            "additional_properties": "object",
                                            "any_of": ["object", "object", "object"],
                                            "items": "object",
                                            "property_names": "string",
                                            "type": "object",
                                        }
                                    },
                                },
                                "name": "name",
                                "type": "type",
                                "inputs": {"foo": "string"},
                                "save_to_memory_as": "save_to_memory_as",
                            },
                            {
                                "config": {
                                    "node_metadata": ["string", "string", "string"],
                                    "num_workers": 0,
                                    "type_hints": {
                                        "foo": {
                                            "additional_properties": "object",
                                            "any_of": ["object", "object", "object"],
                                            "items": "object",
                                            "property_names": "string",
                                            "type": "object",
                                        }
                                    },
                                },
                                "name": "name",
                                "type": "type",
                                "inputs": {"foo": "string"},
                                "save_to_memory_as": "save_to_memory_as",
                            },
                        ],
                    }
                },
            },
        )
        assert_matches_type(object, config, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncAgents) -> None:
        response = await async_client.configs.with_raw_response.create(
            config={
                "plan": [
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                ]
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = await response.parse()
        assert_matches_type(object, config, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncAgents) -> None:
        async with async_client.configs.with_streaming_response.create(
            config={
                "plan": [
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                    {"workflow_name": "workflow_name"},
                ]
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = await response.parse()
            assert_matches_type(object, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncAgents) -> None:
        config = await async_client.configs.retrieve(
            "config_id",
        )
        assert_matches_type(ConfigRetrieveResponse, config, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncAgents) -> None:
        response = await async_client.configs.with_raw_response.retrieve(
            "config_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = await response.parse()
        assert_matches_type(ConfigRetrieveResponse, config, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncAgents) -> None:
        async with async_client.configs.with_streaming_response.retrieve(
            "config_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = await response.parse()
            assert_matches_type(ConfigRetrieveResponse, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncAgents) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_id` but received ''"):
            await async_client.configs.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncAgents) -> None:
        config = await async_client.configs.list(
            config_type="workflow",
        )
        assert_matches_type(ConfigListResponse, config, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncAgents) -> None:
        config = await async_client.configs.list(
            config_type="workflow",
            account_id="account_id",
        )
        assert_matches_type(ConfigListResponse, config, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncAgents) -> None:
        response = await async_client.configs.with_raw_response.list(
            config_type="workflow",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = await response.parse()
        assert_matches_type(ConfigListResponse, config, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncAgents) -> None:
        async with async_client.configs.with_streaming_response.list(
            config_type="workflow",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = await response.parse()
            assert_matches_type(ConfigListResponse, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncAgents) -> None:
        config = await async_client.configs.delete(
            "config_id",
        )
        assert_matches_type(ConfigDeleteResponse, config, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncAgents) -> None:
        response = await async_client.configs.with_raw_response.delete(
            "config_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = await response.parse()
        assert_matches_type(ConfigDeleteResponse, config, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncAgents) -> None:
        async with async_client.configs.with_streaming_response.delete(
            "config_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = await response.parse()
            assert_matches_type(ConfigDeleteResponse, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncAgents) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_id` but received ''"):
            await async_client.configs.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_execute(self, async_client: AsyncAgents) -> None:
        config = await async_client.configs.execute(
            config_id="config_id",
            messages=[
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
            ],
            metadata={},
            run_id="run_id",
            session_id="session_id",
        )
        assert_matches_type(object, config, path=["response"])

    @parametrize
    async def test_method_execute_with_all_params(self, async_client: AsyncAgents) -> None:
        config = await async_client.configs.execute(
            config_id="config_id",
            messages=[
                {
                    "content": "content",
                    "role": "assistant",
                    "uuid": "UUID",
                },
                {
                    "content": "content",
                    "role": "assistant",
                    "uuid": "UUID",
                },
                {
                    "content": "content",
                    "role": "assistant",
                    "uuid": "UUID",
                },
            ],
            metadata={},
            run_id="run_id",
            session_id="session_id",
            id="id",
            concurrent=True,
            stream=True,
        )
        assert_matches_type(object, config, path=["response"])

    @parametrize
    async def test_raw_response_execute(self, async_client: AsyncAgents) -> None:
        response = await async_client.configs.with_raw_response.execute(
            config_id="config_id",
            messages=[
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
            ],
            metadata={},
            run_id="run_id",
            session_id="session_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = await response.parse()
        assert_matches_type(object, config, path=["response"])

    @parametrize
    async def test_streaming_response_execute(self, async_client: AsyncAgents) -> None:
        async with async_client.configs.with_streaming_response.execute(
            config_id="config_id",
            messages=[
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
                {
                    "content": "content",
                    "role": "assistant",
                },
            ],
            metadata={},
            run_id="run_id",
            session_id="session_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = await response.parse()
            assert_matches_type(object, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_execute(self, async_client: AsyncAgents) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_id` but received ''"):
            await async_client.configs.with_raw_response.execute(
                config_id="",
                messages=[
                    {
                        "content": "content",
                        "role": "assistant",
                    },
                    {
                        "content": "content",
                        "role": "assistant",
                    },
                    {
                        "content": "content",
                        "role": "assistant",
                    },
                ],
                metadata={},
                run_id="run_id",
                session_id="session_id",
            )

    @parametrize
    async def test_method_retrieve_details(self, async_client: AsyncAgents) -> None:
        config = await async_client.configs.retrieve_details(
            "config_id",
        )
        assert_matches_type(ConfigRetrieveDetailsResponse, config, path=["response"])

    @parametrize
    async def test_raw_response_retrieve_details(self, async_client: AsyncAgents) -> None:
        response = await async_client.configs.with_raw_response.retrieve_details(
            "config_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        config = await response.parse()
        assert_matches_type(ConfigRetrieveDetailsResponse, config, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve_details(self, async_client: AsyncAgents) -> None:
        async with async_client.configs.with_streaming_response.retrieve_details(
            "config_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            config = await response.parse()
            assert_matches_type(ConfigRetrieveDetailsResponse, config, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve_details(self, async_client: AsyncAgents) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `config_id` but received ''"):
            await async_client.configs.with_raw_response.retrieve_details(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel

__all__ = ["ConfigRetrieveDetailsResponse", "Edge", "Input", "Node"]


class Edge(BaseModel):
    from_node: str

    to_node: str


class Input(BaseModel):
    name: str

    type: str


class Node(BaseModel):
    id: str

    config: object

    name: str

    type: str


class ConfigRetrieveDetailsResponse(BaseModel):
    edges: List[Edge]

    inputs: List[Input]

    nodes: List[Node]
